#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase, Client
from django.contrib.auth.models import User
from django.utils import timezone

from opps.articles.models import Post, Link
from opps.channels.models import Channel


class TemplateNameTest(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )
        self.channel = Channel.objects.create(
            name='test channel',
            slug='test-channel',
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )
        self.post = Post.objects.create(
            headline=u'a simple headline',
            short_title=u'a simple short title',
            title=u'a simple title',
            hat=u'a simple hat',
            channel=self.channel,
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )

    def test_get_template_name_basic(self):
        response = self.client.get(self.post.get_absolute_url())
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.template_name,
            ['containers/test-channel/a-simple-title/detail.html',
             'containers/test-channel/post_detail.html',
             'containers/post_detail.html',
             'containers/test-channel/detail.html',
             'containers/detail.html'])

    def test_get_template_name_channel_with_father(self):
        channel = Channel.objects.create(
            name='child',
            slug='child',
            parent=self.channel,
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )
        self.post.channel = channel
        self.post.save()

        response = self.client.get(self.post.get_absolute_url())
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.template_name,
            ['containers/test-channel/child/a-simple-title/detail.html',
             'containers/test-channel/child/post_detail.html',
             'containers/test-channel/post_detail.html',
             'containers/post_detail.html',
             'containers/test-channel/child/detail.html',
             'containers/test-channel/detail.html',
             'containers/detail.html'])


class LinkResponseToRedirecTest(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )
        self.channel = Channel.objects.create(
            name='test channel',
            slug='test-channel',
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )
        self.link = Link.objects.create(
            title=u'a simple title',
            url=u'http://www.oppsproject.org/',
            channel=self.channel,
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )

    def test_redirect(self):
        response = self.client.get(self.link.get_absolute_url())
        self.assertTrue(response)
        self.assertEqual(response.status_code, 302)
        try:
            self.assertEqual(response.items()[2][1],
                             u'http://www.oppsproject.org/')
        except:
            self.assertEqual(response.items()[3][1],
                             u'http://www.oppsproject.org/')


class TestAjaxRequests(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )
        self.channel = Channel.objects.create(
            name='test channel 2',
            slug='test-channel-2',
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )
        self.post = Post.objects.create(
            headline=u'a simple headline 2',
            short_title=u'a simple short title 2',
            title=u'a simple title 2',
            hat=u'a simple hat 2',
            channel=self.channel,
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )

    def test_if_ajax_extends_variable_in_context_is_empty_without_ajax(self):
        response = self.client.get(self.post.get_absolute_url())
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)

    def test_get_ajax_extends_variable_in_context(self):
        response = self.client.get(self.post.get_absolute_url(),
                                   HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['extends_parent'], 'base_ajax.html')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.db import models


def model_choices():
    try:
        return tuple(sorted([
            (u"{0}.{1}".format(app._meta.app_label, app._meta.object_name),
             u"{0} - {1}".format(app._meta.app_label, app._meta.object_name))
            for app in models.get_models() if 'opps.' in app.__module__]))
    except ImportError:
        return tuple([])



# -*- coding: utf-8 -*-

from haystack.views import SearchView

from opps.containers.models import Container
from opps.views.generic.list import ListView
from opps.views.generic.detail import DetailView


class ContainerList(ListView):
    model = Container


class ContainerDetail(DetailView):
    model = Container


class Search(SearchView):
    def get_results(self):
        return self.form.search().order_by('-date_available')



# -*- coding: utf-8 -*-

from django import template
from django.conf import settings

register = template.Library()


@register.simple_tag(takes_context=True)
def get_rss_link(context):
    try:
        rss_template = settings.OPPS_RSS_LINK_TEMPLATE
        rss_path = '#'
        if 'channel' in context:
            channel = context['channel']
            # TODO: Now by design homepage will always be named home
            # in future use get_homepage to determine the home page.
            if channel['level'] == 0 and channel['slug'] == 'home':
                rss_path = 'http://{0}/rss'.format(
                    context['site'].domain)
            else:
                rss_path = 'http://{0}/{1}/rss'.format(
                    context['site'].domain,
                    channel['long_slug']
                )
        else:
            raise Exception()
        return rss_template.format(rss_path)
    except:
        return ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .conf import settings


def Db(*args, **kwarg):
    _import = settings.OPPS_DB_ENGINE.split('.')[-1]
    _from = u".".join(settings.OPPS_DB_ENGINE.split('.')[:-1])
    call = getattr(__import__(_from, fromlist=[_import]), _import)
    return call(*args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.utils import timezone

from opps.containers.models import Container
from opps.channels.models import Channel

from opps.containers.templatetags.container_tags import \
    get_containers_by, get_container_by_channel, filter_queryset_by, \
    exclude_queryset_by


class ExcludeQuerysetByTest(TestCase):
    def setUp(self):
        User = get_user_model()
        user = User.objects.create(username=u'test', password='test')
        site = Site.objects.filter(name=u'example.com').get()
        channel = Channel.objects.create(name=u'Home', slug=u'home',
                                         description=u'home page',
                                         site=site, user=user)

        Container.objects.create(title=u'test',
                                 user=user,
                                 published=True,
                                 site=site,
                                 channel=channel)
        Container.objects.create(title=u'test 2',
                                 user=user,
                                 site=site,
                                 channel=channel)

    def test_tag(self):
        container = Container.objects.all()
        exclude = exclude_queryset_by(container, published=True)
        self.assertTrue(exclude)
        self.assertEqual(len(container), 2)
        self.assertEqual(len(exclude), 1)


class FilterQuerysetByTest(TestCase):
    def setUp(self):
        User = get_user_model()
        user = User.objects.create(username=u'test', password='test')
        site = Site.objects.filter(name=u'example.com').get()
        channel = Channel.objects.create(name=u'Home', slug=u'home',
                                         description=u'home page',
                                         site=site, user=user)

        Container.objects.create(title=u'test',
                                 user=user,
                                 published=True,
                                 site=site,
                                 channel=channel)
        Container.objects.create(title=u'test 2',
                                 user=user,
                                 site=site,
                                 channel=channel)

    def test_tag(self):
        container = Container.objects.all()
        filter1 = filter_queryset_by(container, published=True)
        self.assertTrue(filter1)
        self.assertEqual(len(container), 2)
        self.assertEqual(len(filter1), 1)


class GetContainerByTest(TestCase):
    def setUp(self):
        User = get_user_model()
        user = User.objects.create(username=u'test', password='test')
        site = Site.objects.filter(name=u'example.com').get()
        channel = Channel.objects.create(name=u'Home', slug=u'home',
                                         description=u'home page',
                                         site=site, user=user)

        self.container = Container.objects.create(title=u'test',
                                                  user=user,
                                                  published=True,
                                                  site=site,
                                                  channel=channel)

    def test_tag(self):
        container = get_containers_by(title=u'test')
        self.assertTrue(container)
        self.assertEqual(container[0], self.container)
        self.assertEqual(len(container), 1)

    def test_more_filter(self):
        container = get_containers_by(title=u'test',
                                      user__username=u'test')
        self.assertTrue(container)
        self.assertEqual(container[0].id, 1)
        self.assertEqual(container[0].user.username, u'test')


class GetContainerByChannelTest(TestCase):
    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.channel = Channel.objects.create(name=u'Home', slug=u'home',
                                              description=u'home page',
                                              site=self.site, user=self.user)
        self.channel2 = Channel.objects.create(
            name=u'Home2', slug=u'home2',
            description=u'home page2',
            site=self.site, user=self.user)

        self.channel3 = Channel.objects.create(
            name=u'Home3', slug=u'home3',
            description=u'home page3',
            parent=self.channel2,
            site=self.site, user=self.user)

        self.count_container_create = [1, 2, 3, 4, 5]

        for i in self.count_container_create:
            Container.objects.create(title=u'test {0}'.format(i),
                                     user=self.user,
                                     published=True,
                                     site=self.site,
                                     channel=self.channel)
        Container.objects.create(title=u'test channel 2',
                                 user=self.user,
                                 published=True,
                                 site=self.site,
                                 channel=self.channel2)

    def test_tag_one_channel(self):
        get_container = get_container_by_channel('home')
        self.assertEqual(len(get_container), len(self.count_container_create))
        for c, i in zip(self.count_container_create[::-1], get_container):
            self.assertEqual(i.slug, "test-{0}".format(c))

    def test_tag_another_channel(self):
        get_container = get_container_by_channel('home2')
        self.assertEqual(len(get_container), 1)
        self.assertEqual(get_container[0].slug, "test-channel-2")

    def test_get_container_recursive(self):
        for i in self.count_container_create:
            Container.objects.create(title=u'test 3 {0}'.format(i),
                                     user=self.user,
                                     published=True,
                                     site=self.site,
                                     channel=self.channel3)

        get_container = get_container_by_channel('home2')
        self.assertEqual(len(get_container), 6)

    def test_get_channel_with_magicdate(self):
        date = timezone.now() - timezone.timedelta(days=30)
        for i in self.count_container_create:
            Container.objects.create(title=u'test 3 {0}'.format(i),
                                     user=self.user,
                                     published=True,
                                     site=self.site,
                                     channel=self.channel,
                                     date_available=date
                                     )

        containers = get_container_by_channel('home',
                                              magic_date='30 days ago')

        self.assertEqual(containers.count(), len(self.count_container_create))



# -*- coding: utf-8 -*-
import operator

from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.core.cache import cache
from django.utils import timezone
from django.conf import settings
from django.template.defaultfilters import slugify
from django.contrib.sites.models import Site

from polymorphic import PolymorphicModel
from polymorphic.showfields import ShowFieldContent

from opps.core.cache import _cache_key
from opps.core.models import Publishable, Slugged, Channeling, Imaged
from opps.core.tags.models import Tagged
from opps.db.models.fields import JSONField
from opps.boxes.models import BaseBox

from .managers import ContainerManager
from .signals import shorturl_generate, delete_container
from .tasks import check_mirror_channel, check_mirror_site


class Container(PolymorphicModel, ShowFieldContent, Publishable, Slugged,
                Channeling, Imaged, Tagged):
    title = models.CharField(_(u"Title"), max_length=140, db_index=True)
    hat = models.CharField(
        _(u"Hat"),
        max_length=140,
        null=True, blank=True,
    )
    short_url = models.URLField(
        _(u"Short URL"),
        null=True, blank=True,
    )
    child_class = models.CharField(
        _(u'Child class'),
        max_length=30,
        null=True, blank=True,
        db_index=True
    )
    child_module = models.CharField(
        _(u'Child module'),
        max_length=120,
        null=True, blank=True,
        db_index=True
    )
    child_app_label = models.CharField(
        _(u'Child app label'),
        max_length=30,
        null=True, blank=True,
        db_index=True
    )
    show_on_root_channel = models.BooleanField(
        _(u"Show on root channel?"),
        default=True
    )
    source = models.CharField(
        _('Source'),
        null=True, blank=True,
        max_length=255)
    json = JSONField(_(u"Customized"),
                     null=True, blank=True)

    objects = ContainerManager()

    def __unicode__(self):
        return u"{0}".format(self.get_absolute_url())

    def __repr__(self):
        val = self.__unicode__()
        if isinstance(val, str):
            return val
        elif not isinstance(val, unicode):
            val = unicode(val)
        return val.encode('utf8')

    class Meta:
        ordering = ['-date_available']
        verbose_name = _(u'Container')
        verbose_name_plural = _(u'Containers')
        unique_together = ("site", "channel", "slug")

    def save(self, *args, **kwargs):
        if not self.channel_name:
            self.channel_name = self.channel.name
        if not self.channel_long_slug:
            self.channel_long_slug = self.channel.long_slug
        self.child_class = self.__class__.__name__
        self.child_module = self.__class__.__module__
        self.child_app_label = self._meta.app_label
        if self.slug == u"":
            self.slug = slugify(self.title)

        models.signals.post_save.connect(shorturl_generate,
                                         sender=self.__class__)
        super(Container, self).save(*args, **kwargs)
        if settings.OPPS_MIRROR_CHANNEL and (
            self.mirror_channel or self.mirror_site)\
                and self.child_class != u"Mirror":
            check_mirror_channel.delay(
                container=self, Mirror=Mirror)
            check_mirror_site.delay(
                container=self, Mirror=Mirror)

    def get_absolute_url(self):
        if self.channel.homepage:
            return u"/{0}.html".format(self.slug)
        return u"/{0}/{1}.html".format(self.channel_long_slug, self.slug)

    @classmethod
    def get_children_models(cls):
        children = models.get_models()
        return [model for model in children
                if (model is not None and
                    issubclass(model, cls) and
                    model is not cls)]

    def get_thumb(self):
        return self.main_image

    @property
    def search_category(self):
        """for use in search result"""
        return _(self.child_class)

    def get_http_absolute_url(self):
        return u"http://{0}{1}".format(self.site_domain,
                                       self.get_absolute_url())
    get_http_absolute_url.short_description = _(u'Get HTTP Absolute URL')

    def recommendation(self, child_class=False, query_slice=[None, 10]):

        if not child_class:
            child_class = self.child_class

        now = timezone.now()
        start = now - timezone.timedelta(
            days=settings.OPPS_RECOMMENDATION_RANGE_DAYS
        )

        cachekey = _cache_key(
            u'{0}-recommendation'.format(self.__class__.__name__),
            self.__class__, self.site_domain,
            u"{0}-{1}-{2}".format(child_class, self.channel_long_slug,
                                  self.slug))
        getcache = cache.get(cachekey)
        if getcache:
            return getcache

        containers = Container.objects.filter(
            site_domain=self.site_domain,
            child_class=child_class,
            channel_long_slug=self.channel_long_slug,
            date_available__range=(start, now),
            published=True
        ).exclude(pk=self.pk)

        tag_list = []
        if self.tags:
            tag_list = [t for t in self.tags.split(',')[:3]]
            containers = containers.filter(
                reduce(operator.or_, (Q(tags__contains=tag) for tag in
                                      tag_list)),
            )
        containers = containers.distinct().order_by(
            '-date_available')[slice(*query_slice)]

        _list = [a for a in containers]

        cache.set(cachekey, _list, 3600)
        return _list

    def inbox(self, containerbox=None):
        obj = ContainerBoxContainers.objects
        if containerbox:
            return obj.get(container=self.id,
                           containerbox__slug=containerbox)
        return obj.filter(container=self.id)

    def custom_fields(self):
        import json
        if not self.json:
            return
        return json.loads(self.json)


class ContainerImage(models.Model):
    container = models.ForeignKey(
        'containers.Container',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Container'),
    )
    order = models.PositiveIntegerField(_(u'Order'), default=0)
    image = models.ForeignKey(
        'images.Image',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Image'),
    )

    caption = models.CharField(
        _(u"Caption"),
        max_length=255,
        blank=True,
        null=True
    )

    class Meta:
        verbose_name = _(u'Container image')
        verbose_name_plural = _(u'Container images')
        ordering = ('order',)

    def __unicode__(self):
        if self.image:
            return u"{0}".format(self.image.title)
        return u'Id:{0} - Order:{1}'.format(self.id, self.order)


class ContainerBox(BaseBox):

    title = models.CharField(
        _(u"Title"),
        null=True,
        blank=True,
        max_length=140,
    )
    title_url = models.CharField(
        _(u"Title Link"),
        null=True,
        blank=True,
        max_length=250,
    )
    main_image = models.ForeignKey(
        'images.Image',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Main Image'),
    )
    main_image_caption = models.CharField(
        _(u"Main Image Caption"),
        max_length=4000,
        blank=True,
        null=True,
        help_text=_(u'Maximum characters 4000'),
    )
    containers = models.ManyToManyField(
        'containers.Container',
        null=True, blank=True,
        verbose_name=_(u'Container'),
        related_name='containerbox_containers',
        through='containers.ContainerBoxContainers'
    )
    queryset = models.ForeignKey(
        'boxes.QuerySet',
        null=True, blank=True,
        related_name='containerbox_querysets',
        verbose_name=_(u'Query Set')
    )

    content_group = models.CharField(
        _(u"Content Group"),
        max_length=255,
        default='default',
        help_text=_(u"Boxes in the same group do not allow repetitions")
    )

    class Meta:
        verbose_name = _(u'Container box')
        verbose_name_plural = _(u'Containers boxes')

    def __init__(self, *args, **kwargs):
        """
        to avoid re-execution of methods
        its results are cached in a local storage
        per instance cache
        """
        super(ContainerBox, self).__init__(*args, **kwargs)
        if not hasattr(self, 'local_cache'):
            self.local_cache = {}

    @property
    def has_content(self):
        # TODO: should check start/end available_dates
        if self.containerboxcontainers_set.exists():
            return True
        qs = self.get_queryset(use_local_cache=False)
        if qs and qs.exists():
            return True

    def ordered_containers(self, field='order', exclude_ids=None):
        cached = self.local_cache.get('ordered_containers', None)
        if cached:
            return cached

        exclude_ids = exclude_ids or []

        now = timezone.now()
        fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
        if not fallback:
            qs = self.containers.filter(
                models.Q(containerboxcontainers__date_end__gte=now) |
                models.Q(containerboxcontainers__date_end__isnull=True),
                published=True,
                date_available__lte=now,
                containerboxcontainers__date_available__lte=now
            ).exclude(
                id__in=exclude_ids
            ).order_by('containerboxcontainers__order').distinct()
        else:

            site_master = cache.get("site_master")

            if not site_master:
                site_master = Site.objects.order_by('id')[0]
                cache.set('site_master', site_master, 3600)

            boxes = [self]

            if fallback and site_master.pk != self.site_id:
                try:
                    master_box = self.__class__.objects.get(
                        site=site_master, slug=self.slug
                    )
                    boxes.insert(0, master_box)
                except self.__class__.DoesNotExist:
                    pass

            qs = ContainerBoxContainers.objects.filter(
                models.Q(date_end__gte=now) |
                models.Q(date_end__isnull=True),
                models.Q(container__published=True,
                         container__date_available__lte=now) |
                models.Q(container__isnull=True),
                containerbox__in=boxes,
                date_available__lte=now,
            ).exclude(
                container__id__in=exclude_ids
            ).order_by('-containerbox__site__id', 'order').distinct()

        self.local_cache['ordered_containers'] = qs
        return qs

    def ordered_box_containers(self, exclude_ids=None):

        fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
        if fallback:
            return self.ordered_containers(exclude_ids=exclude_ids)

        cached = self.local_cache.get('ordered_box_containers')
        if cached:
            return cached

        exclude_ids = exclude_ids or []

        now = timezone.now()
        qs = self.containerboxcontainers_set.filter(
            models.Q(date_end__gte=now) |
            models.Q(date_end__isnull=True),
            date_available__lte=now
        ).exclude(
            container__id__in=exclude_ids
        ).order_by('order').distinct()

        self.local_cache['ordered_box_containers'] = qs
        return qs

    def get_containers_queryset(self):
        if self.queryset:
            return self.queryset.get_queryset()
        else:
            return self.ordered_containers()

    def get_queryset(self, exclude_ids=None, use_local_cache=True):
        cached = self.local_cache.get('get_queryset')
        if use_local_cache and cached:
            return cached

        queryset = self.queryset and self.queryset.get_queryset(
            content_group=self.content_group,
            exclude_ids=exclude_ids,
            use_local_cache=use_local_cache
        )
        if use_local_cache:
            self.local_cache['get_queryset'] = queryset
        return queryset


class ContainerBoxContainers(models.Model):
    URL_TARGET_CHOICES = (
        ('_blank', _(u'Load in a new window')),
        ('_self', _(u'Load in the same frame as it was clicked')),
        ('_parent', _(u'Load in the parent frameset')),
        ('_top', _(u'Load in the full body of the window'))
    )

    containerbox = models.ForeignKey(
        'containers.ContainerBox',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Container Box'),
    )
    container = models.ForeignKey(
        'containers.Container',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Container'),
    )

    order = models.PositiveIntegerField(_(u'Order'), default=0)
    aggregate = models.BooleanField(_(u'Aggregate container'), default=False)
    highlight = models.BooleanField(_(u'Highlight container'), default=False)
    date_available = models.DateTimeField(_(u"Date available"),
                                          default=timezone.now, null=True)
    date_end = models.DateTimeField(_(u"End date"), null=True, blank=True)

    title = models.CharField(_(u"Title"), max_length=140,
                             null=True, blank=True)

    hat = models.CharField(
        _(u"Hat"),
        max_length=140,
        null=True, blank=True,
    )

    main_image = models.ForeignKey(
        'images.Image',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Main Image'),
    )
    main_image_caption = models.CharField(
        _(u"Main Image Caption"),
        max_length=4000,
        blank=True,
        null=True,
        help_text=_(u'Maximum characters 4000'),
    )

    url = models.CharField(_(u"URL"), max_length=255, null=True, blank=True)
    url_target = models.CharField(_(u"URL Target"), max_length=255,
                                  choices=URL_TARGET_CHOICES, default="_self",
                                  null=True, blank=True)

    class Meta:
        ordering = ('order',)
        verbose_name = _(u'Article box articles')
        verbose_name_plural = _(u'Article boxes articles')
        ordering = ('order', 'aggregate',)

    def __unicode__(self):
        if self.container:
            return u"{0}-{1}".format(self.containerbox.slug,
                                     self.container.slug)
        else:
            return u"{0}".format(self.containerbox.slug)

    def clean(self):
        if not self.container and not self.url:
            raise ValidationError(_(u'Please select a Container or insert a '
                                    u'URL!'))

        if self.container and not self.container.published:
            raise ValidationError(_(u'Article not published!'))


class Mirror(Container):
    container = models.ForeignKey('containers.Container',
                                  related_name='containers_mirror',
                                  verbose_name=_(u'Container'))

    class Meta:
        verbose_name = _(u'Mirror')
        verbose_name_plural = _(u'Mirrors')

models.signals.post_delete.connect(delete_container, sender=Container)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.conf import settings

from opps.core.widgets import OppsEditor
from opps.channels.models import Channel

from .models import FlatPage
from opps.core.admin import apply_opps_rules
from opps.contrib.multisite.admin import AdminViewPermission
from opps.images.generate import image_url


class FlatPageAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(FlatPageAdminForm, self).__init__(*args, **kwargs)
        self.fields['channel'].required = False

    def clean_channel(self):
        if self.cleaned_data['channel']:
            return self.cleaned_data["channel"]
        return Channel.objects.get_homepage(site=self.cleaned_data['site'])

    class Meta:
        model = FlatPage
        widgets = {'content': OppsEditor()}


@apply_opps_rules('flatpages')
class FlatPageAdmin(AdminViewPermission):
    form = FlatPageAdminForm
    prepopulated_fields = {"slug": ["title"]}
    readonly_fields = ['get_http_absolute_url', 'short_url', 'image_thumb']
    list_display = ['title', 'site', 'channel_long_slug', 'published',
                    'date_available']
    raw_id_fields = ['main_image', 'channel']

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'title', 'slug', 'get_http_absolute_url',
                       'short_url')}),
        (_(u'Content'), {
            'fields': ('headline', 'content', ('main_image', 'image_thumb'))}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('channel', 'published', 'global_page',
                       'date_available')}),
    )

    def image_thumb(self, obj):
        if obj.main_image:
            return u'<img width="60px" height="60px" src="{0}" />'.format(
                image_url(obj.main_image.archive.url, width=60, height=60))
        return _(u'No Image')
    image_thumb.short_description = _(u'Thumbnail')
    image_thumb.allow_tags = True

    def save_model(self, request, obj, form, change):
        if getattr(obj, 'pk', None) is None:
            obj.user = get_user_model().objects.get(pk=request.user.pk)
            obj.site = Site.objects.get(pk=settings.SITE_ID)
            if not obj.channel:
                obj.channel = Channel.objects.get_homepage(site=obj.site)
        obj.save()

admin.site.register(FlatPage, FlatPageAdmin)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Field'
        db.create_table(u'fields_field', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=255)),
            ('application', self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
        ))
        db.send_create_signal(u'fields', ['Field'])

        # Adding model 'Option'
        db.create_table(u'fields_option', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('field', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fields.Field'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=140)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'fields', ['Option'])

        # Adding model 'FieldOption'
        db.create_table(u'fields_fieldoption', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('field', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fields.Field'])),
            ('option', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fields.Option'])),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
        ))
        db.send_create_signal(u'fields', ['FieldOption'])


    def backwards(self, orm):
        # Deleting model 'Field'
        db.delete_table(u'fields_field')

        # Deleting model 'Option'
        db.delete_table(u'fields_option')

        # Deleting model 'FieldOption'
        db.delete_table(u'fields_fieldoption')


    models = {
        u'fields.field': {
            'Meta': {'object_name': 'Field'},
            'application': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'})
        },
        u'fields.fieldoption': {
            'Meta': {'ordering': "['-order']", 'object_name': 'FieldOption'},
            'field': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fields.Field']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'option': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fields.Option']"}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'})
        },
        u'fields.option': {
            'Meta': {'object_name': 'Option'},
            'field': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fields.Field']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '140'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['fields']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import random
from datetime import datetime

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone


from opps.core.models import Publishable, Slugged
from opps.core.tags.models import Tagged


def get_file_path(instance, filename):
    ext = filename.split('.')[-1]
    filename = u"{0}-{1}.{2}".format(random.getrandbits(32),
                                     instance.slug[:100], ext)
    d = datetime.now()
    folder = u"archives/{0}".format(d.strftime("%Y/%m/%d/"))
    return os.path.join(folder, filename)


class Archive(Publishable, Slugged):

    title = models.CharField(_(u"Title"), max_length=140, db_index=True)
    archive = models.FileField(upload_to=get_file_path,
                               max_length=255,
                               verbose_name=_(u'Archive'),
                               null=True,
                               blank=True)
    archive_link = models.URLField(_(u"Archive URL"),
                                   max_length=255,
                                   null=True,
                                   blank=True)
    description = models.TextField(_(u"Description"), null=True, blank=True)

    source = models.CharField(
        _('Source'),
        null=True, blank=True,
        max_length=255
    )

    def clean(self):
        items = [self.archive, self.archive_link]
        if not any(items):
            raise ValidationError(_(u"You must fill archive or archive URL"))
        if all(items):
            raise ValidationError(_(u"Cannot set archive and archive URL"))

    class Meta:
        verbose_name = _(u'Archive')
        verbose_name_plural = _(u'Archives')
        unique_together = ['site', 'slug']
        abstract = True

    def __unicode__(self):
        return u"{0}-{1}".format(self.site, self.slug)

    def get_absolute_url(self):
        if self.date_available <= timezone.now() and self.published:
            return self.archive_link or self.archive.url
        return u""


class File(Archive, Tagged):

    class Meta:
        verbose_name = _(u'File')
        verbose_name_plural = _(u'Files')



# -*- coding: utf-8 -*-

from django.utils import timezone

from opps.api import BaseHandler

from .models import Container, ContainerBox


class Handler(BaseHandler):
    allowed_methods = ('GET',)

    def read(self, request):
        filters = request.GET.dict()
        filters['date_available__lte'] = timezone.now()
        filters['published'] = True
        if 'paginate_limit' in filters:
            del filters['paginate_limit']

        return self.model.objects.filter(
            **filters)[self._page(request):self._limit(request)]


class ContainerHandler(Handler):
    model = Container


class ContainerBoxHandler(Handler):
    model = ContainerBox
    fields = (
        'name',
        'slug',
        'title',
        'title_url',
        'channel',
        ('containers', ())
    )
    exclude = ['queryset']



from django import forms
from django.conf import settings
from django.template.loader import render_to_string


class MultipleUpload(forms.FileInput):
    def render(self, name, value, attrs=None):
        value = ""
        if value:
            _value = "{0}{1}".format(settings.MEDIA_URL, value)
        return render_to_string(
            "admin/opps/images/multiupload.html",
            {"name": name, "value": _value, "STATIC_URL": settings.STATIC_URL}
        )


class CropExample(forms.TextInput):
    def render(self, name, value, attrs=None):
        return render_to_string(
            "admin/opps/images/cropexample.html",
            {
                "name": name, "value": value,
                'STATIC_URL': settings.STATIC_URL,
                "THUMBOR_SERVER": settings.THUMBOR_SERVER,
                "THUMBOR_MEDIA_URL": settings.THUMBOR_MEDIA_URL,
                "THUMBOR_ENABLED": settings.THUMBOR_ENABLED
            }
        )



# -*- coding: utf-8 -*-

from django import template
from django.conf import settings
from opps.core.models import Config

register = template.Library()

CONFIG_DICT = getattr(settings, 'OPPS_GET_CONFIG_DICT', None)


@register.assignment_tag
def get_config(key, app_label=None, **kwargs):
    """
    {% load config %}
    {% get_config 'key_slug' %}
    {% get_config 'key_slug' app_label='promos'  %}
    {% get_config 'color' app_label='channels' channel__long_slug='/home' %}

    Also works
    {% get_config app_label='opps.polls' key='key_slug' %}
    """
    if CONFIG_DICT:
        try:
            channel = kwargs['channel__long_slug']
            value = CONFIG_DICT.get(channel, {}).get(key, None)
            if value:
                return value
        except:
            pass

    if app_label in ['none', 'null', 'None']:
        try:
            del kwargs['app_label']
        except:
            pass

    return Config.get_value(key, **kwargs)


@register.simple_tag
def get_configs(app_label, key_group, **kwargs):
    """
    {% load config %}
    {% get_configs 'polls' 'key_group_slug' %}
    {% get_configs 'infographics' 'key_group_slug' %}
    {% get_configs 'promos' 'key_group_slug' %}
    """

    if app_label in ['none', 'null', 'None']:
        try:
            del kwargs['app_label']
        except:
            pass

    return Config.get_values(key_group, **kwargs)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from django.contrib.auth import get_user_model

User = get_user_model()


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'ContainerBox.main_image'
        db.add_column(u'containers_containerbox', 'main_image',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['images.Image'], null=True, on_delete=models.SET_NULL, blank=True),
                      keep_default=False)

        # Adding field 'ContainerBox.main_image_caption'
        db.add_column(u'containers_containerbox', 'main_image_caption',
                      self.gf('django.db.models.fields.CharField')(max_length=4000, null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'ContainerBox.main_image'
        db.delete_column(u'containers_containerbox', 'main_image_id')

        # Deleting field 'ContainerBox.main_image_caption'
        db.delete_column(u'containers_containerbox', 'main_image_caption')


    models = {
        u'%s.%s' % (User._meta.app_label, User._meta.module_name): {
            'Meta': {'object_name': User.__name__},
        },
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'boxes.queryset': {
            'Meta': {'object_name': 'QuerySet'},
            'channel': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['channels.Channel']", 'null': 'True', 'blank': 'True'}),
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'db_index': 'True'}),
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'filters': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'limit': ('django.db.models.fields.PositiveIntegerField', [], {'default': '7'}),
            'mirror_site': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'boxes_queryset_mirror_site'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['sites.Site']"}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '140'}),
            'order': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'site_domain': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site_iid': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '150'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['%s.%s']" % (User._meta.app_label, User._meta.object_name)})
        },
        u'channels.channel': {
            'Meta': {'ordering': "['name', 'parent__id', 'published']", 'unique_together': "(('site', 'long_slug', 'slug', 'parent'),)", 'object_name': 'Channel'},
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'db_index': 'True'}),
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'homepage': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'include_in_main_rss': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'layout': ('django.db.models.fields.CharField', [], {'default': "'default'", 'max_length': '250', 'db_index': 'True'}),
            u'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'long_slug': ('django.db.models.fields.SlugField', [], {'max_length': '250'}),
            'mirror_site': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'channels_channel_mirror_site'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['sites.Site']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '60'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "'subchannel'", 'null': 'True', 'to': u"orm['channels.Channel']"}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            u'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'show_in_menu': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'site_domain': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site_iid': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '150'}),
            u'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['%s.%s']" % (User._meta.app_label, User._meta.object_name) })
        },
        u'containers.container': {
            'Meta': {'ordering': "['-date_available']", 'unique_together': "(('site', 'channel_long_slug', 'slug'),)", 'object_name': 'Container'},
            'channel': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['channels.Channel']"}),
            'channel_long_slug': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'channel_name': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '140', 'null': 'True', 'blank': 'True'}),
            'child_app_label': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'child_class': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'child_module': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '120', 'null': 'True', 'blank': 'True'}),
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'db_index': 'True'}),
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'hat': ('django.db.models.fields.CharField', [], {'max_length': '140', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'images': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['images.Image']", 'null': 'True', 'through': u"orm['containers.ContainerImage']", 'blank': 'True'}),
            'json': ('opps.db.models.fields.jsonf.JSONField', [], {'null': 'True', 'blank': 'True'}),
            'main_image': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "u'containers_container_mainimage'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['images.Image']"}),
            'main_image_caption': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'mirror_channel': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'containers_container_mirror_channel'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['channels.Channel']"}),
            'mirror_site': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'containers_container_mirror_site'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['sites.Site']"}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'polymorphic_containers.container_set'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'short_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'show_on_root_channel': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'site_domain': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site_iid': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '150'}),
            'source': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'tags': ('django.db.models.fields.CharField', [], {'max_length': '4000', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '140', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['%s.%s']" % (User._meta.app_label, User._meta.object_name)})
        },
        u'containers.containerbox': {
            'Meta': {'object_name': 'ContainerBox'},
            'channel': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['channels.Channel']"}),
            'channel_long_slug': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'channel_name': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '140', 'null': 'True', 'blank': 'True'}),
            'containers': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'containerbox_containers'", 'to': u"orm['containers.Container']", 'through': u"orm['containers.ContainerBoxContainers']", 'blank': 'True', 'symmetrical': 'False', 'null': 'True'}),
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'db_index': 'True'}),
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'main_image': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['images.Image']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'main_image_caption': ('django.db.models.fields.CharField', [], {'max_length': '4000', 'null': 'True', 'blank': 'True'}),
            'mirror_channel': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'containers_containerbox_mirror_channel'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['channels.Channel']"}),
            'mirror_site': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'containers_containerbox_mirror_site'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['sites.Site']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '140'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'queryset': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'containerbox_querysets'", 'null': 'True', 'to': u"orm['boxes.QuerySet']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'site_domain': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site_iid': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '150'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '140', 'null': 'True', 'blank': 'True'}),
            'title_url': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['%s.%s']" % (User._meta.app_label, User._meta.object_name)})
        },
        u'containers.containerboxcontainers': {
            'Meta': {'ordering': "('order', 'aggregate')", 'object_name': 'ContainerBoxContainers'},
            'aggregate': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'container': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['containers.Container']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'containerbox': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['containers.ContainerBox']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True'}),
            'date_end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'hat': ('django.db.models.fields.CharField', [], {'max_length': '140', 'null': 'True', 'blank': 'True'}),
            'highlight': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'main_image': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['images.Image']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'main_image_caption': ('django.db.models.fields.CharField', [], {'max_length': '4000', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '140', 'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        u'containers.containerimage': {
            'Meta': {'ordering': "('order',)", 'object_name': 'ContainerImage'},
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'container': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['containers.Container']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['images.Image']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'})
        },
        u'containers.mirror': {
            'Meta': {'ordering': "['-date_available']", 'object_name': 'Mirror', '_ormbases': [u'containers.Container']},
            'container': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'containers_mirror'", 'to': u"orm['containers.Container']"}),
            u'container_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['containers.Container']", 'unique': 'True', 'primary_key': 'True'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'images.image': {
            'Meta': {'object_name': 'Image'},
            'archive': ('django.db.models.fields.files.FileField', [], {'max_length': '255'}),
            'crop_example': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'crop_x1': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'crop_x2': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'crop_y1': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'crop_y2': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'db_index': 'True'}),
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fit_in': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'flip': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'flop': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'halign': ('django.db.models.fields.CharField', [], {'default': 'False', 'max_length': '6', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mirror_site': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'images_image_mirror_site'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['sites.Site']"}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'site_domain': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site_iid': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '150'}),
            'smart': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'source': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'tags': ('django.db.models.fields.CharField', [], {'max_length': '4000', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '140', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['%s.%s']" % (User._meta.app_label, User._meta.object_name)}),
            'valign': ('django.db.models.fields.CharField', [], {'default': 'False', 'max_length': '6', 'null': 'True', 'blank': 'True'})
        },
        u'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['containers']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.conf import settings

from opps.contrib.feeds.views import (ContainerFeed, ChannelFeed,
                                      ContainerAtomFeed, ChannelAtomFeed)
from opps.core.tags.views import TagList
from opps.core.cache import cache_page

from .views import ContainerList, ContainerDetail
from .views import Search


urlpatterns = patterns(
    '',
    url(r'^$', ContainerList.as_view(), name='home'),

    url(r'^(rss|feed)$', cache_page(settings.OPPS_CACHE_EXPIRE)(
        ContainerFeed()), name='feed'),

    url(r'^atom$', cache_page(settings.OPPS_CACHE_EXPIRE)(
        ContainerAtomFeed()), name='atom_feed'),

    url(r'^search/', Search(), name='search'),

    url(r'^tag/(?P<tag>[\w//-]+)$',
        cache_page(settings.OPPS_CACHE_EXPIRE)(
            TagList.as_view()), name='tag_open'),

    url(r'^(?P<long_slug>[\w\b//-]+)/(rss|feed)$',
        cache_page(settings.OPPS_CACHE_EXPIRE)(
            ChannelFeed()), name='channel_feed'),

    url(r'^(?P<long_slug>[\w\b//-]+)/atom$',
        cache_page(settings.OPPS_CACHE_EXPIRE)(
            ChannelAtomFeed()), name='channel_atom_feed'),

    url(r'^(?P<channel__long_slug>[\w//-]+)/(?P<slug>[\w-]+)\.html$',
        cache_page(settings.OPPS_CACHE_EXPIRE_DETAIL)(
            ContainerDetail.as_view()), name='open'),

    url(r'^(?P<channel__long_slug>[\w\b//-]+)/$',
        cache_page(settings.OPPS_CACHE_EXPIRE_LIST)(
            ContainerList.as_view()), name='channel'),

    url(r'^(?P<slug>[\w-]+)\.html$',
        cache_page(settings.OPPS_CACHE_EXPIRE_DETAIL)(
            ContainerDetail.as_view()), name='homepage_open'),

)



# -*- encoding: utf-8 -*-
import string
import random

from django.test import TestCase
from django.db import models
from django.contrib.auth.models import User

from opps.articles.models import Article, Post, Album
from opps.articles.models import PostRelated, Link
from opps.channels.models import Channel

from opps.core.tags.models import Tag
from opps.core.managers import PublishableManager


class ArticleFields(TestCase):

    def test_headline(self):
        field = Article._meta.get_field_by_name('headline')[0]
        self.assertEqual(field.__class__, models.TextField)

    def test_short_title(self):
        field = Article._meta.get_field_by_name('short_title')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class PostFields(TestCase):

    def test_content(self):
        field = Post._meta.get_field_by_name('content')[0]
        self.assertEqual(field.__class__, models.TextField)
        self.assertFalse(field.null)
        self.assertFalse(field.blank)

    def test_albums(self):
        field = Post._meta.get_field_by_name('albums')[0]
        self.assertEqual(field.__class__, models.ManyToManyField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_related_posts(self):
        field = Post._meta.get_field_by_name('related_posts')[0]
        self.assertEqual(field.__class__, models.ManyToManyField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class PostRelatedFields(TestCase):

    def test_post(self):
        field = PostRelated._meta.get_field_by_name(u"post")[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_related(self):
        field = PostRelated._meta.get_field_by_name(u"related")[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class LinkFields(TestCase):

    def test_url(self):
        field = Link._meta.get_field_by_name(u"url")[0]
        self.assertEqual(field.__class__, models.URLField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class PostCreation(TestCase):

    def setUp(self):
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )

        self.channel = Channel.objects.create(
            name='test channel',
            user=self.user,
        )

        self.post = Post.objects.create(
            headline=u'a simple headline',
            short_title=u'a simple short title',
            title=u'a simple title',
            hat=u'a simple hat',
            channel=self.channel,
            user=self.user,
        )

    def test_post_fields(self):
        self.assertTrue(self.post.headline, u'a simple headline')
        self.assertTrue(self.post.short_title, u'a simple short title')
        self.assertTrue(self.post.title, u'a simple title')
        self.assertTrue(self.post.hat, u'a simple hat')
        self.assertTrue(self.post.channel, self.channel)
        self.assertTrue(self.post.user, self.user)

    def test_multiple_tags_in_post(self):
        self.post.tags = u','.join(self._gen_tags(100))
        self.post.save()
        self.assertTrue(Tag.objects.count(), 100)

    def test_raiser_tags(self):
        self.post.tags = u"opps, cms, , post"
        self.post.save()
        self.assertEqual([t.name for t in self.post.get_tags()],
                         ['opps', 'cms', 'post'])

    def _gen_tags(self, length):
        tags = []
        for i in range(length):
            tag = u''.join(random.choice(string.ascii_uppercase)
                          for x in range(12))
            tags.append(tag)
        return tags


class LinkCreation(TestCase):
    def setUp(self):
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )
        self.channel = Channel.objects.create(
            name='test channel',
            user=self.user,
        )
        self.link = Link.objects.create(
            user=self.user,
            channel=self.channel,
            url='http://www.google.com',
            headline=u'a beatiful headline',
            short_title=u'a simple short title',
            title=u'a simple title',
            hat=u'a simple hat',
        )

    def test_link_fields(self):
        self.assertTrue(self.link.url, u'http://www.google.com')
        self.assertTrue(self.link.channel, self.channel)
        self.assertTrue(self.link.user, self.user)
        self.assertTrue(self.link.short_title, u'a simple short title')
        self.assertTrue(self.link.title, u'a simple title')
        self.assertTrue(self.link.hat, u'a simple hat')

    def test_is_local(self):
        self.assertFalse(self.link.is_local())

    def test_is_subdomain(self):
        self.assertFalse(self.link.is_subdomain())


class AlbumCreation(TestCase):
    def setUp(self):
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )
        self.channel = Channel.objects.create(
            name='test channel',
            user=self.user,
        )
        self.album = Album.objects.create(
            user=self.user,
            channel=self.channel,
            headline=u'a beatiful headline',
            short_title=u'a simple short title',
            title=u'a simple title',
            hat=u'a simple hat',
        )

    def test_album_fields(self):
        self.assertTrue(self.album.channel, self.channel)
        self.assertTrue(self.album.user, self.user)
        self.assertTrue(self.album.short_title, u'a simple short title')
        self.assertTrue(self.album.title, u'a simple title')
        self.assertTrue(self.album.hat, u'a simple hat')


class PostPublishableManager(TestCase):
    def setUp(self):
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )

        self.channel = Channel.objects.create(
            name='test channel',
            user=self.user,
        )

        self.post = Post.objects.create(
            headline=u'a simple headline',
            short_title=u'a simple short title',
            title=u'a simple title',
            hat=u'a simple hat',
            channel=self.channel,
            user=self.user,
        )

        from django.db.models import Model

        class ModelYet(Model):
            objects = PublishableManager()
        self.Model = ModelYet


    def test_custom_manager(self):
        self.assertEquals(Post.objects.all_published().count(), 0)

        self.post.published = True
        self.post.save()

        self.assertEquals(Post.objects.all_published().count(), 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json

from datetime import datetime

from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from opps.core.models import Publishable, Channeling


class QuerySet(Publishable):
    name = models.CharField(_(u"Dynamic queryset name"), max_length=140)
    slug = models.SlugField(
        _(u"Slug"),
        db_index=True,
        max_length=150,
        unique=True,
    )

    model = models.CharField(_(u'Model'), max_length=150)
    limit = models.PositiveIntegerField(_(u'Limit'), default=7)
    offset = models.PositiveIntegerField(_(u'Offset'), default=0)
    order_field = models.CharField(
        _(u"Order Field"),
        max_length=100,
        default='id',
        help_text=_(u"Take care, should be an existing field or lookup")
    )
    order = models.CharField(_('Order'), max_length=1, choices=(
        ('-', 'DESC'), ('+', 'ASC')))
    channel = models.ForeignKey(
        'channels.Channel',
        verbose_name=_(u"Channel"),
        blank=True,
        null=True
    )

    recursive = models.BooleanField(
        _("Recursive"),
        help_text=_("Bring the content channels and subchannels (tree)"),
        default=False
    )

    filters = models.TextField(
        _(u'Filters'),
        help_text=_(u'Json format extra filters for queryset'),
        blank=True,
        null=True
    )

    excludes = models.TextField(
        _(u'Excludes'),
        help_text=_(u'Json format for queryset excludes'),
        blank=True,
        null=True
    )

    def __init__(self, *args, **kwargs):
        """
        to avoid re-execution of methods
        its results are cached in a local storage
        per instance cache
        """
        super(QuerySet, self).__init__(*args, **kwargs)
        if not hasattr(self, 'local_cache'):
            self.local_cache = {}

    def __unicode__(self):
        return u"{0} {1} {2}".format(self.name, self.slug, self.model)

    def clean(self):

        if self.filters:
            try:
                json.loads(self.filters)
            except:
                raise ValidationError(_(u'Invalid JSON for filters'))

        if self.excludes:
            try:
                json.loads(self.excludes)
            except:
                raise ValidationError(_(u'Invalid JSON for excludes'))

        try:
            self.get_queryset().all()
        except Exception as e:
            raise ValidationError(
                u'Invalid Queryset: {0}'.format(str(e))
            )

        if self.offset >= self.limit:
            raise ValidationError(_(u'Offset can\'t be equal or higher than'
                                    u'limit'))

        if self.recursive:
            if not self.channel:
                raise ValidationError(_(u"To use recursion (channel) is "
                                        u"necessary to select a channel"))

    def get_queryset(self, content_group='default',
                     exclude_ids=None, use_local_cache=True):
        cached = self.local_cache.get('get_queryset')
        if use_local_cache and cached:
            return cached

        exclude_ids = exclude_ids or []

        _app, _model = self.model.split('.')
        model = models.get_model(_app, _model)

        queryset = model.objects.filter(
            published=True,
            date_available__lte=timezone.now(),
            site=self.site
        ).exclude(child_class='Mirror')

        try:
            if model._meta.get_field_by_name('show_on_root_channel'):
                queryset = queryset.filter(show_on_root_channel=True)
        except:
            pass  # silently pass when FieldDoesNotExists

        if self.channel and not self.channel.homepage:
            if self.recursive:
                channel_long_slug = [self.channel.long_slug]
                channel_descendants = self.channel.get_descendants(
                    include_self=False)
                for children in channel_descendants:
                    channel_long_slug.append(children.long_slug)

                queryset = queryset.filter(
                    channel_long_slug__in=channel_long_slug)
            else:
                queryset = queryset.filter(
                    channel_long_slug=self.channel.long_slug)

        if self.filters:
            filters = json.loads(self.filters)

            for key, value in filters.iteritems():
                if value == "datetime.now()":
                    filters[key] = datetime.now()

            queryset = queryset.filter(**filters)

        if self.excludes:
            excludes = json.loads(self.excludes)

            for key, value in excludes.iteritems():
                if value == "datetime.now()":
                    excludes[key] = datetime.now()

            queryset = queryset.exclude(**excludes)

        # importing here to avoid circular imports
        from opps.containers.models import Container
        if issubclass(model, Container):
            queryset = queryset.exclude(
                id__in=exclude_ids
            )

        order_term = self.order_field or 'id'
        if self.order == '-':
            order_term = "-{0}".format(self.order_field or 'id')

        queryset = queryset.order_by(order_term)

        result = queryset[self.offset:self.limit]
        if use_local_cache:
            self.local_cache['get_queryset'] = result
        return result


class BaseBox(Publishable, Channeling):
    name = models.CharField(_(u"Box name"), max_length=140)

    slug = models.SlugField(
        _(u"Slug"),
        db_index=True,
        max_length=150,
    )

    class Meta:
        abstract = True
        unique_together = ['site', 'channel_long_slug', 'slug']

    def __unicode__(self):
        return u"{0}-{1}".format(self.slug, self.site.name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django.test import TestCase
from django.db import models
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site

from opps.channels.models import Channel
from opps.boxes.models import QuerySet, BaseBox


class BoxesFields(TestCase):

    def test_name(self):
        field = QuerySet._meta.get_field_by_name(u"name")[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)

    def test_slug(self):
        field = QuerySet._meta.get_field_by_name(u"slug")[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertTrue(field.db_index)
        self.assertEqual(field.max_length, 150)
        self.assertTrue(field.unique)

    def test_model(self):
        field = QuerySet._meta.get_field_by_name(u"model")[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 150)

    def test_limit(self):
        field = QuerySet._meta.get_field_by_name(u"limit")[0]
        self.assertEqual(field.__class__, models.PositiveIntegerField)
        self.assertEqual(field.default, 7)

    def test_offset(self):
        field = QuerySet._meta.get_field_by_name(u"offset")[0]
        self.assertEqual(field.__class__, models.PositiveIntegerField)
        self.assertEqual(field.default, 0)

    def test_order(self):
        field = QuerySet._meta.get_field_by_name(u"order")[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 1)
        self.assertEqual(field.choices, (('-', 'DESC'), ('+', 'ASC')))

    def test_channel(self):
        field = QuerySet._meta.get_field_by_name(u"filters")[0]
        self.assertEqual(field.__class__, models.TextField)
        self.assertEqual(
            field.help_text,
            u"Json format extra filters for queryset"
        )
        self.assertTrue(field.blank)
        self.assertTrue(field.null)


class BaseBoxFields(TestCase):

    def test_name(self):
        field = BaseBox._meta.get_field_by_name(u"name")[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)

    def test_slug(self):
        field = BaseBox._meta.get_field_by_name(u"slug")[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertTrue(field.db_index)
        self.assertEqual(field.max_length, 150)


class QuerySetTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.channel = Channel.objects.create(name=u'Home', slug=u'home',
                                              description=u'home page',
                                              site=self.site, user=self.user)
        self.model = u"containers.Container"

        self.filters = json.dumps({"employees": [{"firstName": "Anna",
                                                 "lastName": "Smith"}, ]})
        self.excludes= json.dumps({"employees": [{"lastName": "McDonald"}, ]})


        self.queryset = QuerySet.objects.create(name=u"Query",
                                                slug=u"query",
                                                user=self.user,
                                                model=self.model,
                                                channel=self.channel,
                                                filters=self.filters,
                                                excludes=self.excludes)

    def test_queryset_fields(self):
        self.assertEqual(self.queryset.name, u"Query")
        self.assertEqual(self.queryset.slug, u"query")
        self.assertEqual(self.queryset.user, self.user)
        self.assertEqual(self.queryset.model, self.model)
        self.assertEqual(self.queryset.channel, self.channel)
        self.assertEqual(self.queryset.filters, self.filters)

    def test_have_filters_in_clean_function(self):
        self.assertTrue(self.filters)
        self.assertEqual(json.loads(self.filters), {u'employees': [{
            u'lastName': u'Smith',
            u'firstName': u'Anna'}]})


    def test_have_excludes_in_clean_function(self):
        self.assertTrue(self.excludes)
        self.assertEqual(json.loads(self.excludes), {u'employees': [{
            u'lastName': u'McDonald'},]})


    def test_not_filters_and_excludes_in_clean_function(self):
        invalid = QuerySet.objects.create(name=u"Query Test",
                                          slug=u"query_test",
                                          user=self.user,
                                          model=self.model,
                                          channel=self.channel,
                                          filters=None,
                                          excludes=None)

        self.assertEqual(None, invalid.filters)
        self.assertEqual(None, invalid.excludes)
        self.assertFalse(invalid.filters)
        self.assertFalse(invalid.excludes)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _

from appconf import AppConf


trans_app_label = _('Multisite')


class OppsMultiSiteConf(AppConf):
    ADMIN = False

    class Meta:
        prefix = 'opps_multisite'



# -*- coding: utf-8 -*-
import re
import random

from types import MethodType

from django.http import HttpResponseRedirect
from django.http import SimpleCookie
from django.http import HttpRequest
from django.conf import settings


try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

# Instead changing ``settings.TEMPLATE_DIRS`` on the fly, what could
# not work properly when there is concurrent requests, we use
# thread-local variables to determine the template directory used for
# mobile requests, so the template loader
# ``opps.contrib.mobile.template.Loader`` can be used instead to
# define the right templates in each of your project views.
THREAD_LOCALS = local()


def _set_cookie(self, key, value='', max_age=None, expires=None, path='/',
                domain=None, secure=False):
    self._resp_cookies[key] = value
    self.COOKIES[key] = value
    if max_age is not None:
        self._resp_cookies[key]['max-age'] = max_age
    if expires is not None:
        self._resp_cookies[key]['expires'] = expires
    if path is not None:
        self._resp_cookies[key]['path'] = path
    if domain is not None:
        self._resp_cookies[key]['domain'] = domain
    if secure:
        self._resp_cookies[key]['secure'] = True


def _delete_cookie(self, key, path='/', domain=None):
    self.set_cookie(key, max_age=0, path=path, domain=domain,
                    expires='Thu, 01-Jan-1970 00:00:00 GMT')
    try:
        del self.COOKIES[key]
    except KeyError:
        pass


IGNORE_AGENTS = getattr(settings, 'OPPS_MOBILE_IGNORE_USER_AGENTS', [])

USER_AGENTS_TEST_MATCH = (
    "w3c ", "acs-", "alav", "alca", "amoi", "audi",
    "avan", "benq", "bird", "blac", "blaz", "brew",
    "cell", "cldc", "cmd-", "dang", "doco", "eric",
    "hipt", "inno", "ipaq", "java", "jigs", "kddi",
    "keji", "leno", "lg-c", "lg-d", "lg-g", "lge-",
    "maui", "maxo", "midp", "mits", "mmef", "mobi",
    "mot-", "moto", "mwbp", "nec-", "newt", "noki",
    "xda", "palm", "pana", "pant", "phil", "play",
    "port", "prox", "qwap", "sage", "sams", "sany",
    "sch-", "sec-", "send", "seri", "sgh-", "shar",
    "sie-", "siem", "smal", "smar", "sony", "sph-",
    "symb", "t-mo", "teli", "tim-", "tosh", "tsm-",
    "upg1", "upsi", "vk-v", "voda", "wap-", "wapa",
    "wapi", "wapp", "wapr", "webc", "winw", "winw",
    "xda-",)

USER_AGENTS_TEST_SEARCH = u"(?:%s)" % u'|'.join((
    'up.browser', 'up.link', 'mmp', 'symbian', 'smartphone', 'midp',
    'wap', 'phone', 'windows ce', 'pda', 'mobile', 'mini', 'palm',
    'netfront', 'opera mobi',))

USER_AGENTS_EXCEPTION_SEARCH = u"(?:%s)" % u'|'.join(('ipad',))

HTTP_ACCEPT_REGEX = re.compile("application/vnd\.wap\.xhtml\+xml",
                               re.IGNORECASE)


def is_mobile_agent(request):
    user_agents_test_match = r'^(?:%s)' % '|'.join(
        USER_AGENTS_TEST_MATCH)
    user_agents_test_match_regex = re.compile(
        user_agents_test_match, re.IGNORECASE)
    user_agents_test_search_regex = re.compile(
        USER_AGENTS_TEST_SEARCH, re.IGNORECASE)
    user_agents_exception_search_regex = re.compile(
        USER_AGENTS_EXCEPTION_SEARCH, re.IGNORECASE)

    is_mobile = False

    if 'HTTP_USER_AGENT' in request.META:
        user_agent = request.META['HTTP_USER_AGENT']

        if user_agents_test_search_regex.search(user_agent) and \
           not user_agents_exception_search_regex.search(user_agent):
            is_mobile = True
        else:
            if 'HTTP_ACCEPT' in request.META:
                http_accept = request.META['HTTP_ACCEPT']
                if HTTP_ACCEPT_REGEX.search(http_accept):
                    is_mobile = True

        if not is_mobile:
            if user_agents_test_match_regex.match(user_agent):
                is_mobile = True

        # Check for ignore user agents
        if IGNORE_AGENTS and user_agent in IGNORE_AGENTS:
            is_mobile = False

    return is_mobile


class MobileDetectionMiddleware(object):
    u"""Used django-mobile core

    https://github.com/gregmuellegger/django-mobile/blob/3093a9791e5e812021e49
    3226e5393033115c8bf/django_mobile/middleware.py
    """

    def process_request(self, request):
        is_mobile = is_mobile_agent(request)
        request.is_mobile = is_mobile
        THREAD_LOCALS.template_dirs = settings.TEMPLATE_DIRS_WEB

        if is_mobile and settings.OPPS_CHECK_MOBILE:
            THREAD_LOCALS.template_dirs = settings.TEMPLATE_DIRS_MOBILE
            if settings.OPPS_DOMAIN_MOBILE and \
               request.META.get('HTTP_HOST', '') != \
               settings.OPPS_DOMAIN_MOBILE:
                return HttpResponseRedirect(u"{0}://{1}{2}".format(
                    settings.OPPS_PROTOCOL_MOBILE,
                    settings.OPPS_DOMAIN_MOBILE,
                    request.path
                ))


class MobileRedirectMiddleware(object):
    """
    Allows setting and deleting of cookies from requests in exactly the same
    way as responses.

    request.set_cookie('name', 'value')

    The set_cookie and delete_cookie are exactly the same as the ones built
    into the Django HttpResponse class.

    http://docs.djangoproject.com/en/dev/ref/request-response
        /#django.http.HttpResponse.set_cookie
    """
    def process_request(self, request):

        domain = request.META.get('HTTP_HOST', '')
        mobile_domain = settings.OPPS_DOMAIN_MOBILE

        current_cookie = request.COOKIES.get('template_mode', None)
        template_mode = request.GET.get('template_mode', None)
        THREAD_LOCALS.template_dirs = settings.TEMPLATE_DIRS_WEB

        if hasattr(request, "is_mobile"):
            agent_is_mobile = request.is_mobile
        else:
            agent_is_mobile = is_mobile_agent(request)

        domain_is_mobile = domain == mobile_domain

        request_is_mobile = agent_is_mobile and domain_is_mobile

        if not template_mode and not current_cookie:
            if domain_is_mobile:
                template_mode = u'mobile'
            else:
                return

        if request_is_mobile and template_mode == u'desktop':
            prot = settings.OPPS_PROTOCOL_WEB
            web_domain = settings.OPPS_DOMAIN_WEB
            url = u"{0}://{1}/?template_mode=desktop".format(prot, web_domain)
            return HttpResponseRedirect(url)
        elif not request_is_mobile and template_mode == u'mobile':
            prot = settings.OPPS_PROTOCOL_MOBILE
            url = u"{0}://{1}/?template_mode=mobile".format(prot,
                                                            mobile_domain)

            # set cache prefix randon in mobile device
            settings.CACHE_MIDDLEWARE_KEY_PREFIX = u"opps_site-{0}-{1}".format(
                settings.SITE_ID, random.getrandbits(32))

            return HttpResponseRedirect(url)

        request._resp_cookies = SimpleCookie()
        request.set_cookie = MethodType(_set_cookie, request, HttpRequest)
        request.delete_cookie = MethodType(
            _delete_cookie, request, HttpRequest
        )

        if template_mode:
            request.set_cookie('template_mode', template_mode)
            current_cookie = template_mode

        if current_cookie and current_cookie.strip().lower() == u"mobile":
            THREAD_LOCALS.template_dirs = settings.TEMPLATE_DIRS_MOBILE

    def process_response(self, request, response):
        if hasattr(request, '_resp_cookies') and request._resp_cookies:
            response.cookies.update(request._resp_cookies)

        return response



# -*- coding: utf-8 -*-
from django import template
from django.conf import settings
from django.utils import timezone

from opps.core.utils import get_app_model


register = template.Library()


@register.simple_tag(takes_context=True)
def get_box(context, appname, slug, template_name=None):
    """
    {% load box_tags %}
    {% get_box 'polls' 'box_slug' %}
    """
    model = get_app_model(appname, "Box")
    try:
        box = model.objects.get(site=settings.SITE_ID, slug=slug,
                                date_available__lte=timezone.now(),
                                published=True)
    except model.DoesNotExist:
        box = None

    if template_name:
        t = template.loader.get_template(template_name)
    else:
        t = template.loader.get_template(
            '{0}/{1}_detail.html'.format(appname, model.__name__.lower())
        )
    return t.render(template.Context({'{0}'.format(
        model.__name__.lower()): box, 'slug': slug, 'context': context}))


@register.simple_tag(takes_context=True)
def get_all_box(context, appname, channel_long_slug, template_name=None):
    """
    {% load box_tags %}
    {% get_all_box 'polls' 'channel_slug' %}
    """
    model = get_app_model(appname, "Box")
    boxes = model.objects.filter(site=settings.SITE_ID,
                                 date_available__lte=timezone.now(),
                                 published=True,
                                 channel_long_slug=channel_long_slug)

    if template_name:
        t = template.loader.get_template(template_name)
    else:
        t = template.loader.get_template(
            '{0}/{1}_list.html'.format(appname, model.__name__.lower())
        )

    return t.render(template.Context({'{0}boxes'.format(
        model.__name__.lower()): boxes, 'context': context}))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import uuid
import hmac

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model

try:
    from hashlib import sha1
except ImportError:
    import sha
    sha1 = sha.sha


User = get_user_model()


class ApiKey(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             verbose_name=_(u"User"))
    key = models.CharField(_(u"Key"), max_length=255)
    date_insert = models.DateTimeField(_(u"Date insert"), auto_now_add=True)

    def __unicode__(self):
        return u"{0} for {1}".format(self.key, self.user)

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super(ApiKey, self).save(*args, **kwargs)

    def generate_key(self):
        new_uuid = uuid.uuid4()
        return hmac.new(new_uuid.bytes, digestmod=sha1).hexdigest()

    class Meta:
        verbose_name = _(u"API Key")
        verbose_name_plural = _(u"API Keys")


def create_api_key(sender, **kwargs):
    if kwargs.get('created') is True:
        ApiKey.objects.create(user=kwargs.get('instance'))


if 'opps.api' in settings.INSTALLED_APPS:
    models.signals.post_save.connect(create_api_key, User)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django.http import HttpResponse
from django.views.generic import TemplateView
from django.views.generic.detail import BaseDetailView
from django.views.generic.detail import SingleObjectTemplateResponseMixin


def response_mimetype(request):
    if "application/json" in request.META['HTTP_ACCEPT']:
        return "application/json"
    return "text/plain"


class JSONResponse(HttpResponse):
    """JSON response class."""
    def __init__(self, obj='', json_opts={}, mimetype="application/json",
                 *args, **kwargs):
        content = json.dumps(obj, **json_opts)
        super(JSONResponse, self).__init__(content, mimetype, *args, **kwargs)


class JSONPResponse(HttpResponse):
    """JSONP response class."""
    def __init__(self, obj='', json_opts={}, mimetype="application/jsonp",
                 jsonp_callback='jsonpCallback', *args, **kwargs):
        _json_content = json.dumps(obj, **json_opts)
        content = "{0}({1})".format(jsonp_callback, _json_content)
        super(JSONPResponse, self).__init__(content, mimetype, *args,
                                            **kwargs)


class JSONResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """
    HEADERS = {}

    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        response = HttpResponse(
            self.convert_context_to_json(context),
            content_type='application/json',
            **response_kwargs
        )

        for key, value in self.HEADERS.items():
            response[key] = value

        return response

    def convert_context_to_json(self, context):
        "Convert the context dictionary into a JSON object"
        # Note: This is *EXTREMELY* naive; in reality, you'll need
        # to do much more complex handling to ensure that arbitrary
        # objects -- such as Django model instances or querysets
        # -- can be serialized as JSON.
        return json.dumps(context)


class JSONView(JSONResponseMixin, TemplateView):
    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)


class JSONDetailView(JSONResponseMixin, BaseDetailView):
    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)


class HybridDetailView(JSONResponseMixin,
                       SingleObjectTemplateResponseMixin,
                       BaseDetailView):
    def render_to_response(self, context):
        # Look for a 'format=json' GET argument
        if self.request.GET.get('format') == 'json':
            return self.render_to_json_response(context)
        else:
            return super(HybridDetailView, self).render_to_response(context)



from django import template
from django.utils.translation import ugettext_lazy as _

register = template.Library()


@register.simple_tag
def upload_js(container=None):
    if container:
        container_sources = container.source if container.source else ''
        container_tags = container.tags if container.tags else ''
        title = container.title
    else:
        container_sources = container_tags = title = ''
    return u"""
<!-- The template to display files available for upload -->
<script id="template-upload" type="text/x-tmpl">
{% for (var i=0, file; file=o.files[i]; i++) { %}
    <tr class="template-upload fade">
        <td class="preview"><span class="fade"></span></td>
        <td class="name"><span>{%=file.name%}</span></td>
        <td class="size"><span>{%=o.formatFileSize(file.size)%}</span></td>

        <td class="extra">
        <label>
        """ + unicode(_(u'Title')) + u""": <input type="text"
        name="title" value='""" + \
        title + u"""' required></label>
        <label>""" + unicode(_(u'Caption')) + u""":
            <textarea name="caption" rows="2" colums="1">
                        </textarea></label>
        <label>""" + unicode(_(u'Order')) + u""": <input type="text"
        name="order" value="0"/></label>
        <label>""" + unicode(_(u'Source')) + u""":
            <input type="text" name="source" value='""" + \
        container_sources + u"""'></label>
        <label>""" + unicode(_(u'Tags')) + u""":
            <input type="text" name="tags" value='""" + \
        container_tags + u"""'></label>
        </td>
        {% if (file.error) { %}
            <td class="error" colspan="2">
            <span class="label label-important">
            {%=locale.fileupload.error%}
            </span>
            {%=locale.fileupload.errors[file.error] || file.error%}</td>
        {% } else if (o.files.valid && !i) { %}
            <td>
            <div class="progress progress-success progress-striped active">
                <div class="bar" style="width:0%;"></div>
            </div>
            </td>
            <td class="start">{% if (!o.options.autoUpload) { %}
                <button class="btn btn-success">
                    <i class="icon-upload icon-white"></i>
                    <span>{%=locale.fileupload.start%}</span>
                </button>
            {% } %}</td>
        {% } else { %}
            <td colspan="2"></td>
        {% } %}
        <td class="cancel">{% if (!i) { %}
            <button class="btn btn-warning">
                <i class="icon-ban-circle icon-white"></i>
                <span>{%=locale.fileupload.cancel%}</span>
            </button>
        {% } %}</td>
    </tr>
{% } %}
</script>
<!-- The template to display files available for download -->
<script id="template-download" type="text/x-tmpl">
{% for (var i=0, file; file=o.files[i]; i++) { %}
    <tr class="template-download fade">
        {% if (file.error) { %}
            <td></td>
            <td class="name"><span>{%=file.name%}</span></td>
            <td class="size">
            <span>{%=o.formatFileSize(file.size)%}</span></td>
            <td class="error" colspan="2">
            <span class="label label-important">
            {%=locale.fileupload.error%}</span>
            {%=locale.fileupload.errors[file.error] || file.error%}</td>
        {% } else { %}
            <td class="preview">{% if (file.thumbnail_url) { %}
                <a href="{%=file.url%}" title="{%=file.name%}"
                   rel="gallery" download="{%=file.name%}">
                   <img src="{%=file.thumbnail_url%}"></a>
            {% } %}</td>
            <td class="name">
                <a href="{%=file.url%}" title="{%=file.name%}"
                   rel="{%=file.thumbnail_url&&'gallery'%}"
                   download="{%=file.name%}">{%=file.name%}</a>
            </td>
            <td class="size">
            <span>{%=o.formatFileSize(file.size)%}</span></td>
            <td class="success-message" colspan="2">
                {%=locale.fileupload.success%}
            </td>
        {% } %}
        <!-- td class="delete">
            <button class="btn btn-danger"
            data-type="{%=file.delete_type%}" data-url="{%=file.delete_url%}">
                <i class="icon-trash icon-white"></i>
                <span>{%=locale.fileupload.destroy%}</span>
            </button>
            <input type="checkbox" name="delete" value="1">
        </td -->
    </tr>
{% } %}
</script>
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json

from django.forms import widgets
from django.db.models import Field

from django.utils.safestring import mark_safe
from django.conf import settings


"""
To use ALOHA  configure in settings.py

OPPS_EDITOR = {
    'editor': 'aloha',
    'js': ("//cdn.aloha-editor.org/latest/lib/require.js",),
    "css": ("//cdn.aloha-editor.org/latest/css/aloha.css",)
}

To use redactor install redactor
pip install django-wysiwyg-redactor
or set your own static paths
OPPS_EDITOR = {
    'editor': 'redactor',
    'js': ('redactor/redactor.min.js',),
    "css": ('redactor/css/redactor.css',
               'redactor/css/django_admin.css')
}
"""

CONFIG = {
    'editor': 'tinymce',
    'js': ('/static/tinymce/tinymce.min.js',)
}

USER_CONFIG = getattr(settings, 'OPPS_EDITOR', {})

CONFIG.update(USER_CONFIG)

INIT_JS = {
    'tinymce': """
             <script src='/static/opps_tinymce.js'></script>
             <script src='/static/admin/opps/images/js/oppsfilebrowser.js'>
            </script>
              <script type="text/javascript">
            django.jQuery(document).ready(function(){
            tinymce.init(%s);
            });
              </script>""",

    'redactor': """<script type="text/javascript">
                  django.jQuery(document).ready(function(){
                      $("#%s").redactor(%s);
                  });
                  </script>""",

    'aloha': """<script type="text/javascript">
                     Aloha.ready( function() {
                        var $ = Aloha.jQuery;
                        $('#%s').aloha(%s);
                     });
                </script>"""
}


class OppsEditor(widgets.Textarea):

    class Media:
        js = CONFIG.get('js')
        css = {
            'all': CONFIG.get('css', [])
        }

    def __init__(self, *args, **kwargs):
        self.upload_to = kwargs.pop('upload_to', '')
        self.custom_options = kwargs.pop('editor_options', {})
        self.allow_file_upload = kwargs.pop('allow_file_upload', True)
        self.allow_image_upload = kwargs.pop('allow_image_upload', True)
        super(OppsEditor, self).__init__(*args, **kwargs)

    def get_options(self, **kwargs):
        options = CONFIG.copy()
        for key in ('editor', 'css', 'js'):
            options.pop(key, None)
        options.update(kwargs)
        return json.dumps(options)

    def render(self, name, value, attrs=None):
        html = super(OppsEditor, self).render(name, value, attrs)
        final_attrs = self.build_attrs(attrs)
        id_ = final_attrs.get('id')

        editor = CONFIG.get('editor')

        if editor in ('tinymce',):
            options = self.get_options(selector="#{0}".format(id_))
            html += INIT_JS.get(CONFIG.get('editor')) % options
            html = html.replace('"CustomFileBrowser"', "CustomFileBrowser")
            html = html.replace("---editorid---", id_)
        else:
            if editor in ('aloha',):
                html += """
                <script
                src="http://cdn.aloha-editor.org/latest/lib/aloha.js"
                data-aloha-plugins="common/ui,
                                    common/format,
                                    common/list,
                                    common/link,
                                    common/highlighteditables">
                </script>
                """
            html += INIT_JS.get(editor) % (id_, self.get_options())

        return mark_safe(html)


class OppsEditorField(Field):
    def __init__(self, *args, **kwargs):
        options = kwargs.pop('editor_options', {})
        upload_to = kwargs.pop('upload_to', '')
        allow_file_upload = kwargs.pop('allow_file_upload', True)
        allow_image_upload = kwargs.pop('allow_image_upload', True)
        self.widget = OppsEditor(
            editor_options=options,
            upload_to=upload_to,
            allow_file_upload=allow_file_upload,
            allow_image_upload=allow_image_upload
        )
        super(OppsEditorField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "TextField"

    def formfield(self, **kwargs):
        defaults = {'widget': self.widget}
        defaults.update(kwargs)
        return super(OppsEditorField, self).formfield(**defaults)


if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^opps\.core\.widgets\.OppsEditorField"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.views.generic.list import ListView as DjangoListView
from django.contrib.sites.models import Site, get_current_site
from django.utils import timezone
from django.conf import settings

from opps.views.generic.base import View

from opps.containers.models import Mirror, ContainerBox


class ListView(View, DjangoListView):
    template_name_suffix = ''

    def get_template_list(self, domain_folder="containers"):
        templates = []

        if not self.long_slug:
            templates.append('{0}/none.html'.format(domain_folder))
            return templates

        list_name = 'list'

        if self.template_name_suffix:
            list_name = "{0}{1}".format(list_name, self.template_name_suffix)

        if self.channel:
            # Check layout, change via admin
            if self.channel.layout != u'default':
                list_name = self.channel.layout

            if self.channel.group and self.channel.parent:
                templates.append('{0}/{1}/{2}.html'.format(
                    domain_folder, self.channel.parent.long_slug, list_name))

                if self.request.GET.get('page') and\
                   self.__class__.__name__ not in\
                   settings.OPPS_PAGINATE_NOT_APP:
                    templates.append('{0}/{1}/{2}_paginated.html'.format(
                        domain_folder, self.channel.parent.long_slug,
                        list_name))

            if self.request.GET.get('page') and\
               self.__class__.__name__ not in settings.OPPS_PAGINATE_NOT_APP:
                templates.append('{0}/{1}/{2}_paginated.html'.format(
                    domain_folder, self.channel.long_slug, list_name))

            templates.append('{0}/{1}/{2}.html'.format(
                domain_folder, self.channel.long_slug, list_name))

            for t in self.channel.get_ancestors()[::-1]:
                templates.append('{0}/{1}/{2}.html'.format(
                    domain_folder, t.long_slug, list_name))
                if self.request.GET.get('page') and\
                   self.__class__.__name__ not in\
                   settings.OPPS_PAGINATE_NOT_APP:
                    templates.append('{0}/{1}/{2}_paginated.html'.format(
                        domain_folder, t.long_slug, list_name))

        if self.request.GET.get('page') and\
           self.__class__.__name__ not in settings.OPPS_PAGINATE_NOT_APP:
            templates.append('{0}/{1}_paginated.html'.format(domain_folder,
                                                             list_name))

        templates.append('{0}/{1}.html'.format(domain_folder, list_name))

        return templates

    def get_template_names(self):
        if self.template_name:
            return [self.template_name]

        domain_folder = self.get_template_folder()
        template_list = self.get_template_list(domain_folder)

        if domain_folder != "containers":
            template_list.extend(self.get_template_list())

        return template_list

    def get_queryset(self):
        self.fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
        self.site = get_current_site(self.request)
        self.site_master = Site.objects.order_by('id')[0]
        self.long_slug = self.get_long_slug()

        if not self.long_slug:
            return None

        self.set_channel_rules()

        self.articleboxes = ContainerBox.objects.filter(
            channel__long_slug=self.long_slug)

        is_paginated = self.page_kwarg in self.request.GET

        if not is_paginated:
            for box in self.articleboxes:
                self.excluded_ids.update(
                    [a.pk for a in box.ordered_containers()])

        queryset = super(ListView, self).get_queryset()
        filters = {}
        filters['site_domain__in'] = [self.site.domain]
        filters['channel_long_slug__in'] = self.channel_long_slug
        filters['date_available__lte'] = timezone.now()
        filters['published'] = True
        if self.channel and self.channel.is_root_node() and not is_paginated:
            filters['show_on_root_channel'] = True

        # CONTAINER FILTERS
        if self.request.GET.get('type'):
            filters['child_class'] = self.request.GET.get('type')

        if self.fallback and self.site != self.site_master:
            filters.get('site_domain__in', []).append(self.site_master.domain)
        queryset = queryset.filter(**filters)
        if self.excluded_ids:
            queryset = queryset.exclude(
                pk__in=self.excluded_ids)

        # Excluded mirror on root channel
        mirror_excluded = [mirror.id for mirror in Mirror.objects.filter(
            container__channel_long_slug=self.long_slug)]
        queryset = queryset.exclude(pk__in=mirror_excluded)

        return queryset._clone()



# -*- coding: utf-8 -*-
from django import template
from django.template import Node, TemplateSyntaxError, Variable
from django.conf import settings
from django.utils.translation import ugettext as _
from ..generate import image_url as url


register = template.Library()


class AllImagesCheckPermissionForObjectsNode(Node):
    def __init__(self, obj, name):
        self.obj = Variable(obj)
        self.name = name

    def render(self, context):
        check_published = True

        try:
            user = context['request'].user
            if user.is_staff or user.is_superuser:
                check_published = False
        except:
            pass

        obj = self.obj.resolve(context)
        context[self.name] = obj.all_images(check_published)
        return ''


@register.tag(name='all_images_check_permission')
def all_images_check_permission(parser, token):
    """
    {% all_images_check_permission object as images %}
    """
    try:
        parans = token.split_contents()
    except ValueError:
        raise TemplateSyntaxError(
            _('tag requires exactly two arguments'))
    if len(parans) != 4:
        raise TemplateSyntaxError(
            _('tag requires exactly three arguments'))
    if parans[2] != 'as':
        raise TemplateSyntaxError(
            _("second argument to tag must be 'as'"))
    return AllImagesCheckPermissionForObjectsNode(parans[1], parans[3])


@register.simple_tag
def image_url(image_url, **kwargs):
    return url(image_url=image_url, **kwargs)


@register.simple_tag
def image_obj(image, **kwargs):
    HALIGN_VALUES = ("left", "center", "right")
    VALIGN_VALUES = ("top", "middle", "bottom")

    if image == "" or not image:
        return ""

    if settings.THUMBOR_ENABLED:
        new = {}
        new['flip'] = image.flip
        new['flop'] = image.flop

        if image.halign and image.halign in HALIGN_VALUES:
            new['halign'] = image.halign
        if image.valign and image.valign in VALIGN_VALUES:
            new['valign'] = image.valign

        new['fit_in'] = image.fit_in
        new['smart'] = image.smart

        if image.crop_x1 > 0 or image.crop_x2 > 0 or image.crop_y1 > 0 or \
           image.crop_y2 > 0:
            new['crop'] = ((image.crop_x1, image.crop_y1),
                           (image.crop_x2, image.crop_y2))

        kwargs = dict(new, **kwargs)

    if image.archive_link and image.archive_link != "":
        return url(image_url=image.archive_link, **kwargs)

    return image.image_url(**kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required

from .views import GetImagesView, PopUpImageView

urlpatterns = patterns(
    '',
    url(r'^image_add/',
        login_required(login_url='/admin/')(PopUpImageView.as_view()),
        name='image_add'),
    url(r'^get_images/', GetImagesView.as_view(), name='get_images'),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from opps.core.widgets import OppsEditor
from opps.containers.forms import ContainerAdminForm

from .models import Post, Album, Link


class PostAdminForm(ContainerAdminForm):
    multiupload_link = '/fileupload/image/'

    class Meta:
        model = Post
        widgets = {'content': OppsEditor()}


class AlbumAdminForm(ContainerAdminForm):
    multiupload_link = '/fileupload/image/'

    class Meta:
        model = Album
        widgets = {
            'headline': OppsEditor()
        }


class LinkAdminForm(ContainerAdminForm):
    class Meta:
        model = Link



# -*- coding: utf-8 -*-

from datetime import datetime

from django.conf import settings

from haystack.indexes import Indexable

from opps.containers.search_indexes import ContainerIndex

from .models import Post, Album, Link


migration_date = getattr(settings, 'MIGRATION_DATE', None)
if migration_date:
    m_date = datetime.strptime(migration_date, "%Y-%m-%d").date()
    Post.is_legacy = lambda self: m_date >= self.date_insert.date()
else:
    Post.is_legacy = lambda self: False


class PostIndex(ContainerIndex, Indexable):

    def get_model(self):
        return Post


class AlbumIndex(ContainerIndex, Indexable):
    def get_model(self):
        return Album


class LinkIndex(ContainerIndex, Indexable):

    def get_model(self):
        return Link



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import argparse
from django.conf import settings
from django.core import management


settings.configure()
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Opps CMS bin file')
    parser.add_argument('operation', help='task to be performed',
                        choices=['startproject'])
    parser.add_argument("project_name", help="Project name", type=str)

    args = parser.parse_args()
    if args.operation == 'startproject':
        management.call_command(
            'startproject', args.project_name,
            template='https://github.com/opps/opps-project-template/zipball/'
            'master',
            extensions=('py', 'md', 'dev')
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import QuerySet
from .forms import QuerySetAdminForm
from opps.core.admin import PublishableAdmin
from opps.core.admin import apply_opps_rules


@apply_opps_rules('boxes')
class QuerySetAdmin(PublishableAdmin):
    form = QuerySetAdminForm
    prepopulated_fields = {"slug": ["name"]}
    list_display = ['name', 'date_available', 'published']
    list_filter = ['date_available', 'published']
    raw_id_fields = ['channel']
    readonly_fields = ['data_sample']
    search_fields = ['channel__name', 'channel__long_slug', 'name', 'slug']
    exclude = ('user',)

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'name', 'slug')}),
        (_(u'Rules'), {
            'fields': ('model', 'order', 'order_field',
                       'offset', 'limit', 'channel', 'recursive', 'filters',
                       'excludes')}),
        (_(u'Publication'), {
            'classes': ('extrapretty',),
            'fields': ('published', 'date_available')}),
        (_(u'Data sample'), {
            'classes': ('extrapretty',),
            'fields': ('data_sample',)}),
    )

    def data_sample(self, obj):
        html = []
        qs = obj.get_queryset()
        count = qs.count()

        if count:
            html.append(u'<strong>Items:</strong> {0}<ul>'.format(count))
            for item in qs:
                html.append(u"<li>{0}</li>".format(item))
            html.append(u'</ul>')
        else:
            html.append(u'<ul><li>No data</li></ul>')

        return u''.join(html)
    data_sample.short_description = _(u'Data sample')
    data_sample.allow_tags = True


admin.site.register(QuerySet, QuerySetAdmin)



"""
This file was generated with the customdashboard management command and
contains the class for the main dashboard.

To activate your index dashboard add the following to your settings.py::
    GRAPPELLI_INDEX_DASHBOARD = 'project_name.dashboard.CustomIndexDashboard'
"""

from django.utils.translation import ugettext_lazy as _

from grappelli.dashboard import modules, Dashboard


class CustomIndexDashboard(Dashboard):

    """
    Custom index dashboard for www.
    """
    title = "Opps CMS Dashboard"

    def init_with_context(self, context):
        site = context.get("site")

        # append an app list module for "Applications"
        self.children.append(modules.AppList(
            _('Applications'),
            collapsible=True,
            column=1,
            css_classes=('collapse',),
            exclude=('django.contrib.*',),
        ))

        # append an app list module for "Administration"
        self.children.append(modules.ModelList(
            _('Administration'),
            column=1,
            collapsible=True,
            css_classes=('grp-closed',),
            models=('django.contrib.*',),
        ))

        # append another link list module for "support".
        self.children.append(modules.LinkList(
            _('Media Management'),
            column=2,
            children=[
                {
                    'title': _('FileBrowser'),
                    'url': '/admin/filebrowser/browse/',
                    'external': False,
                },
            ]
        ))

        # append another link list module for "support".
        self.children.append(modules.LinkList(
            _('Support'),
            column=2,
            children=[
                {
                    'title': _('Opps Documentation'),
                    'url': 'http://www.oppsproject.org/',
                    'external': True,
                }
            ]
        ))

        # append a feed module
        self.children.append(modules.Feed(
            _('Latest Publications'),
            column=2,
            feed_url='http://{0}/rss'.format(site.domain),
            limit=10
        ))

        # append a recent actions module
        self.children.append(modules.RecentActions(
            _('Recent Actions'),
            limit=5,
            collapsible=False,
            column=3,
        ))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import Image
from .widgets import CropExample

from opps.core.widgets import OppsEditor


class ImageModelForm(forms.ModelForm):
    crop_example = forms.CharField(label=_('Crop Example'), required=False,
                                   widget=CropExample())
    crop_x1 = forms.CharField(label=_(u'Crop X1'), required=False,
                              widget=forms.HiddenInput())
    crop_x2 = forms.CharField(label=_(u'Crop X2'), required=False,
                              widget=forms.HiddenInput())
    crop_y1 = forms.CharField(label=_(u'Crop Y1'), required=False,
                              widget=forms.HiddenInput())
    crop_y2 = forms.CharField(label=_(u'Crop Y2'), required=False,
                              widget=forms.HiddenInput())

    class Meta:
        model = Image
        widgets = {'description': OppsEditor()}


class PopUpImageForm(ImageModelForm):

    source = forms.CharField(
        required=True,
        label=_(u'Source'),
    )

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user')
        super(PopUpImageForm, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        instance = super(PopUpImageForm, self).save(commit=False)
        if not instance.pk:
            instance.published = True
            instance.user = self.user
            instance.save()
        return instance

    class Meta:
        model = Image
        widgets = {'description': OppsEditor()}
        fields = ('site', 'title', 'archive', 'description', 'tags',
                  'source')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

from piston.resource import Resource

from opps.api import ApiKeyAuthentication

from .api import LoggingHandler


logging = Resource(handler=LoggingHandler,
                   authentication=ApiKeyAuthentication())

urlpatterns = patterns(
    '',
    url(r'^api/contrib/logging/$', logging, {'emitter_format': 'json'}),
)



# -*- coding: utf-8 -*-
import warnings
from threading import current_thread


class FakeRequest(object):
    def __init__(self, *args, **kwargs):
        warnings.warn(
            ("You must include "
             "opps.contrib.middleware.global_request.GlobalRequest"
             " as the last one in MIDDLEWARE_CLASSES")
        )


class GlobalRequest(object):
    """Thread-safe middleware that makes current request available globally

    Reference: https://djangosnippets.org/snippets/2853/

    """
    _requests = {}

    @staticmethod
    def get_request():
        try:
            return GlobalRequest._requests[current_thread()]
        except KeyError:
            return FakeRequest()

    def process_request(self, request):
        GlobalRequest._requests[current_thread()] = request

    def process_response(self, request, response):
        # Cleanup
        thread = current_thread()
        try:
            del GlobalRequest._requests[thread]
        except KeyError:
            pass
        return response


def get_request():
    return GlobalRequest.get_request()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase
from django.contrib.sites.models import Site
from django.contrib.auth import get_user_model

from opps.channels.models import Channel
from opps.channels.forms import ChannelAdminForm


class ChannelFormTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.parent = Channel.objects.create(name=u'Home', slug=u'home',
                                             description=u'home page',
                                             site=self.site, user=self.user)

    def test_init(self):
        """
        Test successful init without data
        """
        form = ChannelAdminForm(instance=self.parent)
        self.assertTrue(isinstance(form.instance, Channel))
        self.assertEqual(form.instance.pk, self.parent.pk)
        self.assertEqual(
            int(form.fields['slug'].widget.attrs['maxlength']), 150)

    def test_default_choices_layout(self):
        """
        Check choices default in layout field
        """
        form = ChannelAdminForm(instance=self.parent)
        self.assertTrue(form.fields['layout'].choices)
        self.assertEqual([c for c in form.fields['layout'].choices],
                         [(u'default', u'Default')])

    def test_readonly_slug(self):
        """
        Check readonly field slug
        """
        form = ChannelAdminForm(instance=self.parent)
        self.assertTrue(form.fields['slug'].widget.attrs['readonly'])

        form_2 = ChannelAdminForm()
        self.assertNotIn('readonly', form_2.fields['slug'].widget.attrs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url, include
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/images/', include('opps.images.urls')),
    url(r'^grappelli/', include('grappelli.urls')),

    url(r'^admin/', include(admin.site.urls)),

    url(r'^fileupload/', include('opps.contrib.fileupload.urls')),

    url(r'^sitemap', include('opps.sitemaps.urls')),

    url(r'^api/', include('opps.api.urls')),

    url(r'^', include('opps.flatpages.urls', namespace='flatpages',
                      app_name='flatpages')),

    url(r'^', include('opps.articles.urls', namespace='articles',
                      app_name='articles')),

    url(r'^', include('opps.containers.urls', namespace='containers',
                      app_name='containers')),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase, Client
from django.contrib.auth.models import User
from django.utils import timezone

from opps.channels.models import Channel


class TestTemplateName(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )
        self.channel = Channel.objects.create(
            name='test channel',
            slug='test-channel',
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )

    def test_get_template_name_basic(self):
        response = self.client.get(self.channel.get_absolute_url())
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.template_name,
            ['containers/test-channel/list.html',
             'containers/list.html'])

    def test_get_template_name_subchannel(self):
        channel = Channel.objects.create(
            name='test subchannel',
            slug='test-subchannel',
            parent=self.channel,
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )

        response = self.client.get(channel.get_absolute_url())
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.template_name,
            ['containers/test-channel/test-subchannel/list.html',
             'containers/test-channel/list.html',
             'containers/list.html'])

    def test_set_custom_not_set_homepage(self):
        response = self.client.get("/")
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.template_name,
            ['containers/none.html'])

    def test_get_template_name_2_subchannel(self):
        channel = Channel.objects.create(
            name='test subchannel',
            slug='test-subchannel',
            parent=self.channel,
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )
        channel2 = Channel.objects.create(
            name='test subchannel2',
            slug='test-subchannel2',
            parent=channel,
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )

        response = self.client.get(channel2.get_absolute_url())
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.template_name,
            ['containers/test-channel/test-subchannel/'
             'test-subchannel2/list.html',
             'containers/test-channel/test-subchannel/list.html',
             'containers/test-channel/list.html',
             'containers/list.html'])


class TestAjaxRequests(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create(
            username='test@test.com',
            email='test@test.com',
            password=User.objects.make_random_password(),
        )
        self.channel = Channel.objects.create(
            name='test channel 2',
            slug='test-channel-2',
            user=self.user,
            published=True,
            date_available=timezone.now(),
        )

    def test_get_ajax_extends_variable_in_context(self):
        response = self.client.get(self.channel.get_absolute_url(),
                                   HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertTrue(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['extends_parent'], 'base_ajax.html')



from django.core.management.base import BaseCommand
from django.core import serializers

from opps.boxes.models import QuerySet
from opps.channels.models import Channel
from opps.containers.models import ContainerBox


class Command(BaseCommand):

    def handle(self, *args, **options):
        models = [Channel, ContainerBox, QuerySet]
        for m in models:
            data = serializers.serialize("json", m.objects.all())
            out = open("opps_{0}.json".format(m.__name__), "w")
            out.write(data)
            out.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from django.db import models
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site

from opps.containers.models import Container
from opps.channels.models import Channel
from opps.fields.models import Field, Option, FieldOption, FIELD_TYPE


class FieldModelTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.channel = Channel.objects.create(name=u'Home', slug=u'home',
                                              description=u'home page',
                                              site=self.site, user=self.user)

        self.container = Container.objects.create(title=u'test',
                                                  user=self.user,
                                                  published=True,
                                                  site=self.site,
                                                  channel=self.channel)


class FieldFields(TestCase):

    def test_name(self):
        field = Field._meta.get_field_by_name('name')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 100)

    def test_slug(self):
        field = Field._meta.get_field_by_name('slug')[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertEqual(field.max_length, 255)
        self.assertFalse(field.null)
        self.assertFalse(field.blank)

    def test_application(self):
        field = Field._meta.get_field_by_name('application')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 255)
        self.assertFalse(field.null)
        self.assertFalse(field.blank)
        self.assertTrue(field.db_index)

    def test_type(self):
        field = Field._meta.get_field_by_name('type')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 15)
        self.assertTrue(field.db_index)
        self.assertTrue(field.choices, FIELD_TYPE)


class OptionFields(TestCase):

    def test_field(self):
        field = Option._meta.get_field_by_name('field')[0]
        self.assertEqual(field.__class__, models.ForeignKey)

    def test_name(self):
        field = Field._meta.get_field_by_name('name')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 100)
        self.assertFalse(field.null)
        self.assertFalse(field.blank)

    def test_slug(self):
        field = Option._meta.get_field_by_name('slug')[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertEqual(field.max_length, 140)
        self.assertFalse(field.null)
        self.assertFalse(field.blank)

    def test_value(self):
        field = Option._meta.get_field_by_name('value')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 255)
        self.assertFalse(field.null)
        self.assertFalse(field.blank)


class FieldOptionFields(TestCase):

    def test_field(self):
        field = FieldOption._meta.get_field_by_name('field')[0]
        self.assertEqual(field.__class__, models.ForeignKey)

    def test_option(self):
        field = FieldOption._meta.get_field_by_name('option')[0]
        self.assertEqual(field.__class__, models.ForeignKey)

    def test_order(self):
        field = FieldOption._meta.get_field_by_name('order')[0]
        self.assertEqual(field.__class__, models.PositiveIntegerField)
        self.assertEqual(field.default, 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django.db import models
from django.utils.translation import ugettext_lazy as _

from opps.core.models import Publishable
from opps.db import Db


NOTIFICATION_TYPE = (
    (u'json', _(u'JSON')),
    (u'text', _(u'Text')),
    (u'html', _(u'HTML')),
)


class Notification(Publishable):
    container = models.ForeignKey('containers.Container')
    channel_long_slug = models.CharField(
        _(u"Channel long slug"),
        max_length=250,
        null=True, blank=True,
        db_index=True,
    )
    slug = models.SlugField(
        _(u"Slug"),
        max_length=150,
        null=True, blank=True,
        db_index=True,
    )

    action = models.CharField(_('Action'), max_length=75,
                              default="message")
    type = models.CharField(_('Type'), max_length=10,
                            choices=NOTIFICATION_TYPE,
                            default='json')
    message = models.TextField(_('Message'))

    def save(self, *args, **kwargs):
        self.channel_long_slug = self.container.channel_long_slug
        self.slug = self.container.slug
        super(Notification, self).save(*args, **kwargs)
        _db = Db(self.container.get_absolute_url(),
                 self.container.id)
        message = self.message
        if self.type == "json":
            message = json.dumps(self.message)
        _db.publish(json.dumps({
            "action": self.action,
            "id": self.id,
            "published": self.published,
            "date": self.date_available.strftime("%D %T"),
            "message": message}))
        _db.close()



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _

from opps.containers.signals import shorturl_generate
from opps.articles.models import Article


class FlatPage(Article):
    show_in_menu = models.BooleanField(_(u"Show in menu?"), default=False)
    global_page = models.BooleanField(_(u"Show in all sites?"), default=False)
    content = models.TextField(_(u"Content"))
    order = models.IntegerField(_(u"Order"), default=0)

    class Meta:
        verbose_name = _(u'Flatpage')
        verbose_name_plural = _(u'Flatpages')

    def get_absolute_url(self):
        return u"/page/{0}".format(self.slug)

    def get_http_absolute_url(self):
        return u"http://{0}{1}".format(self.site.domain,
                                       self.get_absolute_url())
    get_http_absolute_url.short_description = 'URL'


models.signals.post_save.connect(shorturl_generate, sender=FlatPage)



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from opps.core.models import Date, Slugged


class Tag(Date, Slugged):
    name = models.CharField(_(u'Name'), max_length=255, unique=True,
                            db_index=True)

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name)
        super(Tag, self).save(*args, **kwargs)

    __unicode__ = lambda self: self.name

    class Meta:
        verbose_name = _(u'Tag')
        verbose_name_plural = _(u'Tags')
        unique_together = ['slug', 'name']


class Tagged(models.Model):
    tags = models.CharField(_(u'Tags'), max_length=4000, blank=True,
                            null=True,
                            help_text=_(u'A comma-separated list of tags.'))

    def save(self, *args, **kwargs):
        if self.tags:
            # Remove empty and repeated strings on list
            tags = filter(None, set(self.tags.split(',')))
            tags = map(unicode.strip, tags)
            for tag in tags:
                Tag.objects.get_or_create(name=tag)
            self.tags = ','.join(tags)

        super(Tagged, self).save(*args, **kwargs)

    def get_tags(self):
        if self.tags:
            tags = []
            for tag in self.tags.split(','):
                if tag in ["", " "]:
                    continue
                t, created = Tag.objects.get_or_create(name=tag)
                tags.append(t)
            return tags

    class Meta:
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.conf import settings

from opps.core.cache import cache_page

from .views import AlbumList
from .views import AlbumChannelList


urlpatterns = patterns(
    '',
    url(r'^albums/$',
        cache_page(settings.OPPS_CACHE_EXPIRE_LIST)(
            AlbumList.as_view()), name='album_list'),

    url(r'^album/(?P<channel__long_slug>[\w\b//-]+)/$',
        cache_page(settings.OPPS_CACHE_EXPIRE_LIST)(
            AlbumChannelList.as_view()), name='album_channel'),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib.sitemaps import GenericSitemap as DjangoGenericSitemap
from django.contrib.sitemaps import Sitemap as DjangoSitemap
from django.utils import timezone

from opps.containers.models import Container


def InfoDict(googlenews=False):
    containers = Container.objects.filter(date_available__lte=timezone.now(),
                                          published=True)
    if googlenews:
        containers = containers[:1000]
    return {
        'queryset': containers,
        'date_field': 'date_available',
    }


class BaseSitemap(DjangoSitemap):
    priority = 0.6

    def items(self):
        return Container.objects.filter(date_available__lte=timezone.now(),
                                        published=True)

    def lastmod(self, obj):
        return obj.date_available


class GenericSitemap(DjangoGenericSitemap):
    limit = 1000
    priority = 0.6



from django.conf import settings
from django.contrib.sites.models import get_current_site
from django.test import TestCase
from django.test.client import RequestFactory

from opps.channels.context_processors import channel_context


class ChannelContextTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.result = channel_context(self.request)

    def test_site(self):
        expected = get_current_site(self.request)
        self.assertEqual(expected, self.result['site'])

    def test_conf(self):
        self.assertEqual(settings.OPPS_CHANNEL_CONF,
                         self.result['opps_channel_conf_all'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unicodedata


def normalize_tags(tags):
    """ Normalise (normalize) unicode data in Python to remove umlauts,
    accents etc. """

    if not tags:
        return []
    try:
        return [
            unicodedata.normalize(
                'NFKD', tag.lower().strip()
            ).encode('ASCII', 'ignore')
            for tag in tags.split(',')
        ]
    except:
        return [tags]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django.contrib import admin
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from .models import Container, ContainerImage, Mirror
from .models import ContainerBox, ContainerBoxContainers
from .forms import ContainerBoxContainersInlineForm
from opps.core.admin import PublishableAdmin, apply_opps_rules, BaseBoxAdmin
from opps.contrib.multisite.admin import AdminViewPermission
from opps.core.filters import ChannelListFilter, HasQuerySet
from opps.images.generate import image_url
from opps.fields.models import Field, FieldOption


@apply_opps_rules('containers')
class ContainerImageInline(admin.TabularInline):
    model = ContainerImage
    fk_name = 'container'
    raw_id_fields = ['image']
    sortable_field_name = "order"
    actions = None
    extra = 0
    verbose_name = _(u"Container image")
    verbose_name_plural = _(u"Container images")
    fieldsets = [(None, {'fields': ('image', 'image_thumb',
                         'order', 'caption')})]
    ordering = ('order',)
    readonly_fields = ['image_thumb']

    def image_thumb(self, obj):
        if obj.image:
            return u'<img width="60px" height="60px" src="{0}" />'.format(
                image_url(obj.image.archive.url, width=60, height=60))
        return _(u'No Image')
    image_thumb.short_description = _(u'Thumbnail')
    image_thumb.allow_tags = True


@apply_opps_rules('containers')
class ContainerBoxContainersInline(admin.StackedInline):
    model = ContainerBoxContainers
    form = ContainerBoxContainersInlineForm
    fk_name = 'containerbox'
    raw_id_fields = ['container', 'main_image']
    sortable_field_name = "order"
    actions = None
    ordering = ('order',)
    extra = 0
    fieldsets = [(None, {
        'fields': ('container', 'aggregate', 'highlight', 'order',
                   'date_available', 'date_end', 'hat', 'title',
                   'main_image', 'main_image_caption', 'url', 'url_target')})]


@apply_opps_rules('containers')
class ContainerAdmin(PublishableAdmin, AdminViewPermission):
    prepopulated_fields = {"slug": ["title"]}
    readonly_fields = ['get_http_absolute_url', 'short_url',
                       'in_containerboxes', 'image_thumb']
    raw_id_fields = ['main_image', 'channel', 'mirror_channel']
    ordering = ('-date_available',)

    autocomplete_lookup_fields = {
        'fk': ['channel'],
    }

    def get_list_filter(self, request):
        list_filter = super(ContainerAdmin, self).list_filter
        list_filter = [ChannelListFilter] + list(list_filter)
        return list_filter

    def save_model(self, request, obj, form, change):
        super(ContainerAdmin, self).save_model(request, obj, form, change)
        _json = {}
        for field in Field.objects.filter(
                application__contains=obj.__class__.__name__):
            if field.type == 'checkbox':
                for fo in FieldOption.objects.filter(field=field):
                    key = "{0}_{1}".format(field.slug, fo.option.slug)
                    _json[key] = request.POST.get('json_{0}'.format(key), '')
            else:
                _json[field.slug] = request.POST.get(
                    'json_{0}'.format(field.slug), '')

        obj.json = json.dumps(_json)
        obj.save()


@apply_opps_rules('containers')
class ContainerBoxAdmin(BaseBoxAdmin, AdminViewPermission):
    inlines = [ContainerBoxContainersInline]
    raw_id_fields = ['channel', 'queryset', 'main_image']
    list_display = ['name', 'site', 'channel_name', 'date_available',
                    'published']
    save_as = True

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'name', 'slug', 'title', 'title_url',
                       'main_image', 'main_image_caption')}),
        (_(u'Relationships'), {
            'fields': ('channel', 'queryset')}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('content_group', 'published', 'date_available')}),
    )

    autocomplete_lookup_fields = {
        'fk': ['channel'],
    }

    def clean_ended_entries(self, request, queryset):
        now = timezone.now()
        for box in queryset:
            ended = box.containerboxcontainers_containerboxes.filter(
                date_end__lt=now
            )
            if ended:
                ended.delete()
    clean_ended_entries.short_description = _(u'Clean ended containers')

    def get_list_display(self, request):
        list_display = getattr(self, 'list_display', [])
        if request.user.is_superuser:
            return list_display + ['is_dynamic']
        return list_display

    def get_list_filter(self, request):
        list_filter = super(ContainerBoxAdmin, self).list_filter
        if request.user.is_superuser:
            list_filter = [HasQuerySet] + list_filter
        return list_filter

    def is_dynamic(self, obj):
        if obj.queryset:
            return True
        else:
            return False
    is_dynamic.short_description = _(u'Dynamic')
    is_dynamic.boolean = True

    actions = ('clean_ended_entries',)


class HideContainerAdmin(PublishableAdmin, AdminViewPermission):

    list_display = ['image_thumb', 'get_child_class', 'title',
                    'channel_name', 'date_available',
                    'published']
    readonly_fields = ['image_thumb']

    def get_child_class(self, obj):
        return _(obj.child_class)
    get_child_class.short_description = _(u'Child class')

    def get_model_perms(self, *args, **kwargs):
        return {}

    def has_add_permission(self, request):
        return False

    def get_list_filter(self, request):
        list_filter = super(HideContainerAdmin, self).list_filter
        list_filter = [ChannelListFilter] + list(list_filter)
        return list_filter

    def queryset(self, request):
        qs = super(HideContainerAdmin, self).queryset(request)
        # TODO: Document this
        blacklist = getattr(settings, 'OPPS_CONTAINERS_BLACKLIST', [])
        if blacklist:
            qs = qs.exclude(child_class__in=blacklist)
        return qs


admin.site.register(Container, HideContainerAdmin)
admin.site.register(ContainerBox, ContainerBoxAdmin)
admin.site.register(Mirror, HideContainerAdmin)
